/* Author: Jason Leigh
 * Avatar class.
 * (C) 1997 Electronic Visualization Laboratory - Univ Illinois at Chicago
 */

#include "CAVERNst_baseAvatar_c.h++"
#include "_support_hash.h++"
#include <sys/time.h>
#include "lim_logcode.h++"

int CAVERNst_baseAvatarManager_c::itsTrackerDataSize = 0;
int CAVERNst_baseAvatarManager_c::itsHelloDataSize = 0;
int CAVERNst_baseAvatarManager_c::itsAuxDataSize = 0;

// Thread for avatar garbage collector.
CAVERNplus_thread_c *g_cavernst_baseAvatarGarbageCollectThread;

void CAVERNst_baseAvatar_c :: lock()
{
	nexus_mutex_lock(&avatarMutex);
}

void CAVERNst_baseAvatar_c :: unlock()
{
	nexus_mutex_unlock(&avatarMutex);
}


CAVERNst_baseAvatar_c :: CAVERNst_baseAvatar_c(long avID,int trackerDataSize, int helloDataSize, int auxDataSize)
{
  avatarID = avID;
  trackerData = new char[trackerDataSize];
  helloData = new char[helloDataSize];
  auxData = new char[auxDataSize];
  nexus_mutex_init(&avatarMutex, NULL);
  
  bodyDCS = headDCS = handDCS = mouthDCS = avatarDCS = NULL;
  
  bodyX = bodyY = bodyZ = handX = handY = handZ	= -1000;
  bodyAngleY					= 0;
  headAngleX = headAngleY = headAngleZ		= 0;
  handAngleX = handAngleY = handAngleZ		= 0;
  
  //ADDED FOR CONSOLE!!!
  //////////////////////
  tetheredTo = NULL;
  activeTether = 0;
  numTethers = 0;
  accTetherReq = 0;

}

CAVERNst_baseAvatar_c :: ~CAVERNst_baseAvatar_c()
{
	if (trackerData) delete trackerData;
	if (helloData) delete helloData;
	if (auxData) delete auxData;
}

// Defined in cvrn_comm.c++
double cvrnGetTimeInSecs();

CAVERNst_baseAvatarManager_c :: ~CAVERNst_baseAvatarManager_c()
{
	sendBye();

	if (avatarDB) delete avatarDB;
	if (hailingChannel) delete hailingChannel;
	if (trackerChannel) delete trackerChannel;
	if (ahcKey) delete ahcKey;
	if (atcKey) delete atcKey;
	if (hailingChannelLink) delete hailingChannelLink;
	if (trackerChannelLink) delete trackerChannelLink;
	if (selfAvatar) delete selfAvatar;
	if (ahcBuffer) delete ahcBuffer;
	if (atcBuffer) delete atcBuffer;
	if (tempBuffer) delete tempBuffer;
}

double CAVERNst_baseAvatarManager_c :: getTheTimeAccurate()
{
	const double oneMicroSec = 0.000001;
	static struct timeval timeSec;
	static struct timezone tzone;

	gettimeofday(&timeSec, &tzone);

	return (((double) timeSec.tv_sec) +
		(((double) timeSec.tv_usec) * oneMicroSec));

}

void CAVERNst_baseAvatarManager_c :: setTrackerUpdateDelay(double delay)
{
	trackerDelay = delay;
}

void CAVERNst_baseAvatarManager_c :: garbageCollect()
{
	static double lastTick = cvrnGetTimeInSecs();
	double currentTick;

	currentTick= cvrnGetTimeInSecs();


	if ((currentTick - lastTick) > 60) {
		lastTick = currentTick;

		/// Every 60 seconds check for dead avatars.
		int numItems;

		/// Grab a list of avatars
		CAVERNst_baseAvatar_c **list;
		avatarDB->lock();
		list  = avatarDB->buildListOfEntries(&numItems);

		CAVERNst_baseAvatar_c *theAvatar;

		if (numItems) {
			for (int i = 0; i < numItems; i++) {
				theAvatar = list[i];

				/// If there are avatars that have not been refreshed in
				/// 2 minutes purge them.
				if ((cvrnGetTimeInSecs() - theAvatar->time) > 120) {
					avatarDB->remove(theAvatar->avatarID);
					cvrnMesg("CAVERNst_baseAvatarManager_c","Avatar %ld timed out",list[i]->avatarID);

					/// Call user callback
					if (avatarExitCB)
						avatarExitCB(theAvatar);


					delete theAvatar;
				}
			}
		}
		avatarDB->unlock();

		if (numItems) delete list;
	}

}

#if (0)
/* Main thread function for garbage collection.
Every 60 seconds this thread gathers a list of all the avatars and figures
out which ones have not sent any data in 120 seconds. Those are purged
and an exit callback is sent back to the user program.
*/
void *  cavernst_baseAvatarGarbageCollect(void *manager)
{

	CAVERNst_baseAvatarManager_c *theManager = (CAVERNst_baseAvatarManager_c *)manager;

	while(1) {

		/// Check for exit flag caused by a delete of the avatar manager.
		for (int ticker = 0; ticker < 60; ticker++) {
			sleep(1);
			if (theManager->threadStatus){
				/// Let manager know that this thread will now terminate.
				theManager->threadStatus = -1;
				return NULL;
			}
		}

		cvrnPrintf("Past garbage thread loop\n");

		/// Every 60 seconds check for dead avatars.
		int numItems;

		/// Grab a list of avatars
		CAVERNst_baseAvatar_c **list;
		theManager->avatarDB->lock();
		list  = theManager->avatarDB->buildListOfEntries(&numItems);

		CAVERNst_baseAvatar_c *theAvatar;

		if (numItems) {
			for (int i = 0; i < numItems; i++) {
				theAvatar = list[i];

				/// If there are avatars that have not been refreshed in
				/// 2 minutes purge them.
				if ((cvrnGetTimeInSecs() - theAvatar->time) > 120) {
					theManager->avatarDB->remove(theAvatar->avatarID);
					cvrnMesg("CAVERNst_baseAvatarManager_c","Avatar %ld timed out",list[i]->avatarID);

					/// Call user callback
					if (theManager->avatarExitCB)
						theManager->avatarExitCB(theAvatar);


					delete theAvatar;
				}
			}
		}
		theManager->avatarDB->unlock();

		if (numItems) delete list;
	}

}
#endif


#define _CAVERNST_BASE_AVATAR_MAX(x, y) ((x)>(y) ? (x) : (y))


void CAVERNst_baseAvatarManager_c :: applyToAllAvatars(
		int (*userFunc)(long avatarID,
				CAVERNst_baseAvatar_c *,
				void *inParam1, void *inParam2,
				void *inParam3, void *& inOutParam),
		void *inParam1, void *inParam2,
		void *inParam3, void *& inOutParam)
{
	avatarDB->lock();
	avatarDB->applyToAll(userFunc, inParam1, inParam2, inParam3, inOutParam);
	avatarDB->unlock();
}


/* Initialize the avatar manager. It sets up keys for the Avatar hailing channel (AHC) and
the Avatar Tracker Channel (ATC). It also creates channels and links to remote IRBs if
one is specified.
 */
int CAVERNst_baseAvatarManager_c :: init(char* applicationName, CAVERN_irb_c *theIrb,
					 CAVERN_irbId_c *avatarIrbServer,
					 int trackerDataSize, int helloDataSize,
					 int auxDataSize)
{

	if (trackerDataSize < 1) {
		cvrnMesg("CAVERNst_baseAvatarManager_c","Cannot set tracker data size to less than 1 byte.");
		exit(1);
	}

	if (helloDataSize < 1) {
		cvrnMesg("CAVERNst_baseAvatarManager_c","Cannot set hello data size to less than 1 byte.");
		exit(1);
	}

	// Allocate memory for packing and sending hello and trackerdata.
	ahcBufferSize =  1+sizeof(long) + _CAVERNST_BASE_AVATAR_MAX(helloDataSize, auxDataSize);
	ahcBuffer = new char[ahcBufferSize];
	atcBufferSize =  1+sizeof(long) + trackerDataSize;
	atcBuffer = new char[atcBufferSize];
	tempBuffer = new char[helloDataSize];

	requireSendAHCHello = 0;
	itsTrackerDataSize = trackerDataSize;
	itsHelloDataSize = helloDataSize;
	itsAuxDataSize = auxDataSize;

	/// Generate AVATAR ID -----------------------------------------------------
	double theTime = cvrnGetTimeInSecs();

	CAVERN_irbId_c selfId;
	selfId = theIrb->getSelfIRBId();

	/// Avatar ID is composed of self IPaddress, port, and time of day in seconds
	char idStr[256];
	sprintf(idStr,"%s:%u:%lf", selfId.getAddress(), selfId.getPort(),
		theTime);

	/// Then this string is hashed into a unique 4 byte number.
	_support_hashWord_t hashVal = _support_Hash(idStr, strlen(idStr));
	long avatarID = hashVal;

	cvrnMesg("AVATAR INFO","YOUR AVATAR ID IS: %ld",avatarID);

	//-------------------------------------------------------------------------

	// Allocate memory for hash dictionary in which avatars are stored.
	avatarDB = new _support_hashDict<CAVERNst_baseAvatar_c *, long>;
	selfAvatar = new CAVERNst_baseAvatar_c(avatarID,trackerDataSize,
					       helloDataSize,auxDataSize);

	if (!selfAvatar) {
		cvrnMesg("CAVERNst_baseAvatarManager_c","Cannot allocate memory for self avatar.");
		exit(1);
	}


	// Save a pointer to personal IRB.
	itsIrb = theIrb;

	// IRB ID of remote IRB to connect to.
	if (avatarIrbServer) {
		itsAvatarIrbServer = *avatarIrbServer;
		selfServer = 0;
	}

	// If avatarIrbServer is NULL then this client is acting as the server
	// and so it makes no links to a remote IRB.
	else {
		selfServer = 1;
	}

	CAVERN_irb_c::status_t irbStatus;
	CAVERN_irbKeyId_c ahcKeyId, atcKeyId;

	// Define the local AHC and ATC keys.
	char appPath[256];

	// Use the application name to generate a path that
	// is unique for each application so that the keys
	// created for avatar data transport don't get mingled and avatars 
	// for different applications start popping up in the wrong apps.
	sprintf(appPath,"%s_avatar", applicationName);
	ahcKeyId.setPath(appPath);
	ahcKeyId.setName("AHC");
	ahcKey = itsIrb->define(&ahcKeyId,NULL,&irbStatus);
	ahcKey->trigger(ahcKeyCB, this);

	atcKeyId.setPath(appPath);
	atcKeyId.setName("ATC");
	atcKey = itsIrb->define(&atcKeyId,NULL,&irbStatus);
	atcKey->trigger(atcKeyCB, this);

	// If this client is not acting as a server open channels
	if (!selfServer) {

		hailingChannel = itsIrb->createChannel();
		trackerChannel = itsIrb->createChannel();

		CAVERN_irbChannel_c::status_t retStatus;
		hailingChannel->open(&itsAvatarIrbServer, NULL, CAVERN_irbChannel_c::RELIABLE,
				     &retStatus);
		if (retStatus != CAVERN_irbChannel_c::OK) exit(1);

		monitorChannel(hailingChannel, "limbo_AVA_HAIL",10,10);

		trackerChannel->open(&itsAvatarIrbServer, NULL, CAVERN_irbChannel_c::UNRELIABLE,
				     &retStatus);
		if (retStatus != CAVERN_irbChannel_c::OK) exit(1);

		monitorChannel(hailingChannel, "limbo_AVA_TRAC",10,10);

		// Make the links do NO initial synching.
		CAVERN_linkAttrib_c linkAttrib;
		linkAttrib.setInitialSynchType(CAVERN_linkAttrib_c::NONE);
		hailingChannelLink = hailingChannel->link(ahcKey, &ahcKeyId, &linkAttrib);
		if (!hailingChannelLink) {
			cvrnMesg("CAVERNst_baseAvatarManager_c","Unable to create a link for avatar hailing key to remote IRB key.");
			exit(1);
		}
		trackerChannelLink = trackerChannel->link(atcKey, &atcKeyId, &linkAttrib);
		if (!trackerChannelLink) {
			cvrnMesg("CAVERNst_baseAvatarManager_c","Unable to create a link for avatar tracker key to remote IRB key.");
			exit(1);
		}

	}

	/// Start the avatar garbage collection thread
	/*
	g_cavernst_baseAvatarGarbageCollectThread = new CAVERNplus_thread_c;
	
	g_cavernst_baseAvatarGarbageCollectThread->create(cavernst_baseAvatarGarbageCollect,this);
	*/
	return 1;
}

void CAVERNst_baseAvatarManager_c :: sendHello()
{
	putHelloToAHC();
}

void CAVERNst_baseAvatarManager_c :: sendAux()
{
	putAuxToAHC();
}

void CAVERNst_baseAvatarManager_c :: sendTracker()
{
	garbageCollect();

	double theTime = getTheTimeAccurate();
	if ((theTime - lastTimeTrackerSent) > trackerDelay) {
		lastTimeTrackerSent = theTime;
		putTrackerToATC();
	}
}

void CAVERNst_baseAvatarManager_c :: sendBye()
{
	putByeToAHC();
	sleep(3);
}

/* Send a hello message that consists of:
   HELLO, AVATAR ID, HELLO DATA
*/
int CAVERNst_baseAvatarManager_c :: putHelloToAHC()
{
	CAVERNplus_datapack_c datapack;


	datapack.setBuffer(ahcBuffer, ahcBufferSize);
	datapack.packChar(_AVATAR_HELLO_MSG);
	datapack.packLong(selfAvatar->avatarID);
	datapack.pack(selfAvatar->helloData,itsHelloDataSize);

	CAVERN_irbKey_c::status_t status;
	int size = datapack.getBufferFilledSize();
	ahcKey->put(ahcBuffer, &size, &status);

	return 1;
}

int CAVERNst_baseAvatarManager_c :: putAuxToAHC()
{
	CAVERNplus_datapack_c datapack;


	datapack.setBuffer(ahcBuffer, ahcBufferSize);
	datapack.packChar(_AVATAR_AUX_MSG);
	datapack.packLong(selfAvatar->avatarID);
	datapack.pack(selfAvatar->auxData,itsAuxDataSize);

	CAVERN_irbKey_c::status_t status;
	int size = datapack.getBufferFilledSize();
	ahcKey->put(ahcBuffer, &size, &status);

	return 1;
}

/*
  Send Bye message that consists of:
  BYE AVATAR ID
  */
int CAVERNst_baseAvatarManager_c :: putByeToAHC()
{
	CAVERNplus_datapack_c datapack;

	datapack.setBuffer(ahcBuffer, ahcBufferSize);
	datapack.packChar(_AVATAR_BYE_MSG);
	datapack.packLong(selfAvatar->avatarID);

	CAVERN_irbKey_c::status_t status;
	int size = datapack.getBufferFilledSize();
	ahcKey->put(ahcBuffer, &size, &status);

	return 1;
}

/*
  Send tracker data message which consists of:
  AVATAR_TRACKER_HEADER, avatarID, tracker data.
  */
int CAVERNst_baseAvatarManager_c :: putTrackerToATC()
{
	CAVERNplus_datapack_c datapack;

	datapack.setBuffer(atcBuffer, atcBufferSize);
	datapack.packChar(_AVATAR_TRACKER_MSG);
	datapack.packLong(selfAvatar->avatarID);
	datapack.pack(selfAvatar->trackerData, itsTrackerDataSize);

	CAVERN_irbKey_c::status_t status;
	int size;
	size = datapack.getBufferFilledSize();
	atcKey->put(atcBuffer, &size, &status);

	return 1;
}

// Callback for when avatar messages are received over the hailing channel.
void CAVERNst_baseAvatarManager_c :: ahcKeyCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t,
						   CAVERN_irbKey_c *theKey,
						   void* manager)
{

	char *buf;
	int size;
	CAVERNst_baseAvatarManager_c *theAvatarManager;
	theAvatarManager = (CAVERNst_baseAvatarManager_c *) manager;

	theAvatarManager->avatarDB->lock();

	CAVERN_irbKey_c::status_t status;
	theKey->getDirect(buf, &size, &status);
	CAVERNplus_datapack_c datapack;

	if (size == 0) {
		theKey->releaseDirect(&status);
		goto getout;
	}
		
	// Extract command and avatar id
	datapack.setBuffer(buf, size);
	char command = datapack.unpackChar();
	long avatarID = datapack.unpackLong();

	// Look for the avatar.
	CAVERNst_baseAvatar_c *theAvatar;
	theAvatarManager->avatarDB->find(avatarID, theAvatar);

	if (command == _AVATAR_HELLO_MSG) {

		// If avatar does not exist yet make one.
		if (!theAvatar) {

			theAvatar = new CAVERNst_baseAvatar_c(avatarID, itsTrackerDataSize,
							    itsHelloDataSize, itsAuxDataSize);
			if (!theAvatar) {
				cvrnMesg("CAVERNst_baseAvatar_c","Unable to allocate new entry for avatar.");
				theKey->releaseDirect(&status);
				goto getout;
			}

			/// Refresh the timer so that this avatar does not get garbage collected.
			theAvatar->time = cvrnGetTimeInSecs();

			// Store avatar in avatar database
			theAvatarManager->avatarDB->enter(avatarID, theAvatar);
			datapack.unpack(theAvatar->helloData, itsHelloDataSize);
			theKey->releaseDirect(&status);

			cvrnMesg("CAVERNst_baseAvatar_c","NEW AVATAR IN TOWN %d",avatarID);

			// Call the user's callback if any
			if (theAvatarManager->newAvatarCB)
				theAvatarManager->newAvatarCB(theAvatar);

			// Set a flag so that a hello message can be sent out.
			if (command == _AVATAR_HELLO_MSG)
				theAvatarManager->requireSendAHCHello = 1;

		} else {
			/// Refresh the timer so that this avatar does not get garbage collected.
			theAvatar->time = cvrnGetTimeInSecs();

			// Grab the hello data.
			datapack.unpack(theAvatarManager->tempBuffer, itsHelloDataSize);
			theKey->releaseDirect(&status);

			// Is it different from previous value?
			if (bcmp(theAvatarManager->tempBuffer, theAvatar->helloData, itsHelloDataSize)) {

				// Replace with new one.
				memcpy(theAvatar->helloData, theAvatarManager->tempBuffer, itsHelloDataSize);

				// Call user callback.
				if (theAvatarManager->avatarHelloChangeCB)
					theAvatarManager->avatarHelloChangeCB(theAvatar);
			}
		}
	} else if (command == _AVATAR_BYE_MSG) {

		theKey->releaseDirect(&status);

		if (!theAvatar) goto getout;

		theAvatarManager->avatarDB->remove(avatarID);

		if (theAvatarManager->avatarExitCB) theAvatarManager->avatarExitCB(theAvatar);
		delete theAvatar;

		goto getout;
	} else if (command == _AVATAR_AUX_MSG) {
		if (!theAvatar) {
			theKey->releaseDirect(&status);
			goto getout;
		}
		/// Refresh the timer so that this avatar does not get garbage collected.
		theAvatar->time = cvrnGetTimeInSecs();

		datapack.unpack(theAvatar->auxData, itsAuxDataSize);
		theKey->releaseDirect(&status);

		// Call user callback.
		if (theAvatarManager->avatarAuxCB)
			theAvatarManager->avatarAuxCB(theAvatar);

		// LEFT OFF
		// Grab aux data and call callback.
	}

 getout:
	theAvatarManager->avatarDB->unlock();

}

void CAVERNst_baseAvatarManager_c :: setHelloData(char *hello)
{
	memcpy(selfAvatar->helloData,hello, itsHelloDataSize);
}

void CAVERNst_baseAvatarManager_c :: setAuxData(char *aux)
{
	memcpy(selfAvatar->auxData,aux, itsAuxDataSize);
}

void CAVERNst_baseAvatarManager_c :: setTrackerData(char *tracker)
{
	memcpy(selfAvatar->trackerData,tracker, itsTrackerDataSize);
}


// Callback for when avatar tracker data comes in.
void CAVERNst_baseAvatarManager_c :: atcKeyCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t,
						   CAVERN_irbKey_c *theKey,
						   void* manager)
{
	char *buf;
	int size;
	CAVERNst_baseAvatarManager_c *theAvatarManager;
	theAvatarManager = (CAVERNst_baseAvatarManager_c *) manager;

	// This is an opportunity to send out a hello message based on
	// whether someone had already sent us a hello message.
	if (theAvatarManager->requireSendAHCHello) {
		theAvatarManager->putHelloToAHC();
		theAvatarManager->requireSendAHCHello = 0;
	}

	// Grab the command and avatar id.
	CAVERN_irbKey_c::status_t status;
	theKey->getDirect(buf, &size, &status);
	CAVERNplus_datapack_c datapack;

	datapack.setBuffer(buf, size);
	char command = datapack.unpackChar();
	long avatarID = datapack.unpackLong();

	// Look for the avatar.
	CAVERNst_baseAvatar_c *theAvatar;
	theAvatarManager->avatarDB->lock();
	theAvatarManager->avatarDB->find(avatarID, theAvatar);
	theAvatarManager->avatarDB->unlock();

	// If it's not found then ignore the packet.
	if (!theAvatar) {
		//		cvrnPrintf("No avatar\n");
		theKey->releaseDirect (&status);
		return;
	}

	// If it is found then retrieve the avatar tracker data.
	if (command == _AVATAR_TRACKER_MSG) {
		theAvatar->time = cvrnGetTimeInSecs();

		datapack.unpack(theAvatar->trackerData, itsTrackerDataSize);

		// Fire the user's callback
		if (theAvatarManager->avatarTrackerCB)
			theAvatarManager->avatarTrackerCB(theAvatar);

	} 
	theKey->releaseDirect(&status);

}
