#include "lim_global.h++";
#include "My_fpRadar.h++"

/************************************************************************/
/* Performer type data
*/

pfType *My_fpRadar::classType_ = NULL; 

void My_fpRadar::init(void)
{
 if (classType_ == NULL)
	{
        pfDCS::init();	 /* Initialize the parent class */
        classType_ =  new pfType(pfDCS::getClassType(), "My_fpRadar");	
   /* Create the new pfType for this class, based on the parent class's type */
	}
 return;
}
/************************************************************************/


My_fpRadar::My_fpRadar(ItemType it) 
  :Radar(it)
{
  char objName[128];
  sprintf(objName,"two-dimensional radar");
  cvrnPrintf("%s\n", objName);
  setName(objName);
  setTravMask(PFTRAV_ISECT,
	      // Intersection mask
	      0x8,
	      PFTRAV_SELF | PFTRAV_DESCEND | PFTRAV_IS_CACHE,
	      PF_SET);
  setTravMask(PFTRAV_DRAW,
	      // Drawing mask
	      0x0,
	      PFTRAV_SELF | PFTRAV_DESCEND,
	      PF_SET);
  pfGeode *radar;
  pfGeode *you;
  pfVec4 color;
  blips = new pfGroup;
  setType(classType_);  /* set the Performer type of this instance */
  active_ = 1;
  setScale(0.5f);
  color[0]= 1.0f;
  color[1]= 1.0f;
  color[2]= 1.0f;
  color[3]= 0.5f;
  radar = createCircle(1.0f, color,PFSTATE_TRANSPARENCY); 
  addChild(radar);
  color[0]= 1.0f;
  color[1]= 0.0f;
  color[2]= 0.0f;
  color[3]= 1.0f;
  you = createCircle(0.03f, color,PFSTATE_ENHIGHLIGHTING); 
  addChild(you);
  addChild(blips);
  setTrans(-1.5, 1.5, 6);
  users_tracked = 0;
}

pfGeode * My_fpRadar::createCircle(float radius,pfVec4 color, int mode)
{
 pfGeoSet *gset;
 pfGeoState *gstate;
 pfVec4 *colorAttr;
 pfMatrix matrix;
 pfGeode *geode = new pfGeode;
 active_ = 1;
 gset = pfdNewCircle(16,pfGetSharedArena());

 gstate = new pfGeoState;
 gstate->setMode(mode, 1);

 gset->setGState(gstate);

 colorAttr = (pfVec4 *) pfMalloc(sizeof(pfVec4),pfGetSharedArena());
 *colorAttr = color;
 gset->setAttr(PFGS_COLOR4,PFGS_OVERALL,colorAttr,NULL);
 gset->setDrawMode(PFGS_FLATSHADE, 1);
 matrix.makeScale(radius,radius,radius);
 pfdXformGSet(gset,matrix);
 geode->addGSet(gset);
 return geode;
}


void My_fpRadar::addUser(CAVERNst_baseAvatar_c *new_user)
{
  cvrnPrintf("FLAT RADAR ADDING %d\n", new_user->avatarID);
  char name[128];
  pfVec4 color;
  pfDCS *blipDCS = new pfDCS();

  sprintf(name, "%d", new_user->avatarID);

  color[0]= 0.0f;
  color[1]= 1.0f;
  color[2]= 0.0f;
  color[3]= 1.0f;

  pfGeode *blipGeode = createCircle(0.05f, color,PFSTATE_ENHIGHLIGHTING);
  blipDCS->addChild(blipGeode);
  blipDCS->setName(name);
  blipDCS->setTravMask(PFTRAV_DRAW,
		       0x0,
		       PFTRAV_SELF | PFTRAV_DESCEND,
		       PF_SET);
  blips->addChild(blipDCS);
  users_tracked++;
  cvrnPrintf("USER %s ADDED\n", name);
  return;
}

void My_fpRadar::removeUser(CAVERNst_baseAvatar_c *old_user)
{
  char name[128];
  sprintf(name, "%d", old_user->avatarID);

  for(int i = 0; i < blips->getNumChildren(); i++)
    {
      if(strcmp(name, ((pfDCS *)(blips->getChild(i)))->getName()) == 0)
	{ 
	  pfDCS *tempDCS = ((pfDCS*)blips->removeChild(blips->getChild(i))); 
	  pfGeode *tempGeode = ((pfGeode*)tempDCS->removeChild(0));
	  pfDelete (tempGeode);
	  pfDelete (tempDCS);
	  return;
	}      
    }
  
  cvrnPrintf("ERROR: Attempted to remove unknown user from radar!\n");
  return;
}

void My_fpRadar::updateALL() { }
 
void My_fpRadar::updateONLY(char * name, int property) { }

int My_fpRadar::app(pfTraverser *trav)
{
  int avatarNum;//, blipsNum;
  pfVec3 myLocation, difference, avatarPos;
  pfDCS *avatar;
  const char *DCSName;
  char name[128];
  long avatarID;
  CAVERNst_baseAvatar_c *theAvatar;
  if (!active_)
    return pfDCS::app(trav);
  
  avatarNum = g_limAvatarDCS->getNumChildren(); 

  if(avatarNum > 0 && users_tracked > 0)
    {
      //cvrnPrintf("avatarNum = %d\n", avatarNum);
      for (int i=0; i < avatarNum; i++)
	{
	  avatar = (pfDCS*)blips->getChild(i);
	  DCSName = avatar->getName();
	  avatarID = atol(DCSName);
	  g_limAvatarManager->avatarDB->find(avatarID, theAvatar);
	  CAVEGetPosition(CAVE_HEAD, myLocation.vec);
	  avatarPos[0] = theAvatar->bodyX;
	  avatarPos[1] = theAvatar->bodyY;
	  avatarPos[2] = 0;
	  CAVENavConvertWorldToCAVE(avatarPos.vec, difference.vec);
	  if ((difference[0] > -75) && (difference[0] < 75) && 
	      (difference[1] > -75) && (difference[1] < 75))
	    {
	      difference[0] = difference[0]/100.0f;
	      difference[1] =  difference[1]/100.0f;
	      difference[2] = 0.0f;
	      sprintf(name, "%d", avatarID);
	      for(int j=0; j < blips->getNumChildren(); j++)
		{
		  if(strcmp(name, ((pfDCS *)(blips->getChild(j)))->getName()) == 0)
		    {
		      ((pfDCS *)(blips->getChild(j)))->setTrans(difference[0], 
								difference[1], 
								difference[2]);
		      ((pfDCS *)(blips->getChild(j)))->setTravMask(PFTRAV_DRAW, 
								   0xffffffff, 
								   PFTRAV_SELF | 
								   PFTRAV_DESCEND,
								   PF_SET);
		    }
		}
	    }
	}
    }
  
  return pfDCS::app(trav);
}

int My_fpRadar::needsApp(void)
{
 return TRUE;
}


int My_fpRadar::isActive()
{
    return active_;
}
