#ifndef __CATCH_H__
#define __CATCH_H__
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdio.h>
#include <unistd.h>

#define CATCH_ADDRESS     "/tmp/catchThrowSocket"  /* addr to connect */
#define CATCH_SIZE        16384

int catchSocket ;
int catchSocketFileDescriptor ;
char catchBuffer[CATCH_SIZE] ;
int catchBufferPtr ;
int catchDebug = 0 ;

void catchInit() {

  int len ;
  int fromlen ;
  struct sockaddr_un saun, fsaun;
  
  if ((catchSocket = socket(AF_UNIX, SOCK_STREAM, 0)) < 0) {
    perror("catchInit: socket");
    exit(1);
  }
  
  saun.sun_family = AF_UNIX;
  strcpy(saun.sun_path, CATCH_ADDRESS);
  
  unlink(CATCH_ADDRESS);
  len = sizeof(saun.sun_family) + strlen(saun.sun_path);
  
  if (bind(catchSocket, &saun, len) < 0) {
    perror("catchInit: bind");
    exit(1);
  }
  
  if (listen(catchSocket, 5) < 0) {
    perror("catchInit: listen");
    exit(1);
  }
  
  if ((catchSocketFileDescriptor = accept(catchSocket, &fsaun, &fromlen)) < 0) {
    perror("catchInit: accept");
    exit(1);
  }
  
  catchBufferPtr = 0 ;
  
}

int catchData(char *line, int size) {
  
  int n ;
  int i ;
  
  n = recv(catchSocketFileDescriptor, &catchBuffer[catchBufferPtr],
	   CATCH_SIZE-catchBufferPtr, MSG_DONTWAIT) ;
  if (catchDebug) {
    printf("%d bytes received from client\n",n) ;
  }
  if (n>0) {
    catchBufferPtr += n ;
  }

  /* any complete messages to pass back? */

  if (catchDebug) {
    printf("before looking for terminator:\n") ;
    for (i=0; i<catchBufferPtr; i++) {
      printf("catchBuffer[%2.2d]: %c %d\n",i,catchBuffer[i],catchBuffer[i]) ;
    }
  }
  for (size=1; size<catchBufferPtr; size++) {
    if (catchBuffer[size] == '\0') {
      memcpy(line,catchBuffer,size+1) ;
      catchBufferPtr -= size+1 ;
      memmove(catchBuffer,&catchBuffer[size+1],catchBufferPtr) ;
      if (catchDebug) {
	printf("after finding terminator:\n") ;
	for (i=0; i<catchBufferPtr; i++) {
	  printf("catchBuffer[%2.2d]: %c %d\n",i,catchBuffer[i],catchBuffer[i]) ;
	}
      }
      return(size) ;
    }
  }
  /* get here if no terminator */
  return(0) ;
}

void termCatch() {
  close(catchSocket);
  unlink(CATCH_ADDRESS);
}
#endif /* !__CATCH_H__ */




