#include<Performer/pr.h>
#include<unistd.h>
#include "console.h++"
#include "My_tsRadar.h++"
#include "My_fpRadar.h++"
#include "partList.h++"
#include "lim_avatar.h++"
#include "tether.h++"
#include "shareview.h++"

//6 Tether Callbacks
////////////////////
void csTetherCB(CAVERNst_baseAvatar_c *, CAVERNplus_datapack_c *);
void crTetherCB(CAVERNplus_datapack_c *);
void cs_tConfirmCB(CAVERNst_baseAvatar_c *, CAVERNplus_datapack_c *);
void cr_tConfirmCB(CAVERNplus_datapack_c *);
void cs_tDetachCB(CAVERNst_baseAvatar_c *, CAVERNplus_datapack_c *);
void cr_tDetachCB(CAVERNplus_datapack_c *);

//6 ShareView Callbacks
////////////////////
void csShareViewCB(CAVERNst_baseAvatar_c *, CAVERNplus_datapack_c *);
void crShareViewCB(CAVERNplus_datapack_c *);
void cs_sConfirmCB(CAVERNst_baseAvatar_c *, CAVERNplus_datapack_c *);
void cr_sConfirmCB(CAVERNplus_datapack_c *);
void cs_sDetachCB(CAVERNst_baseAvatar_c *, CAVERNplus_datapack_c *);
void cr_sDetachCB(CAVERNplus_datapack_c *);

///////////////////////////////////////////////////
// NOTE:  These two functions can probably share // 
// confirmation callbacks (if not all callbacks) //
///////////////////////////////////////////////////


Tether *tether = NULL;
ShareView *view = NULL;

Console::Console()
{
  active_ = 1;
  pfs_pointer = NULL;
  current_items = new List<ConsoleItem>();
}
Console::~Console()
{
  active_ = 0;
  pfs_pointer = NULL;
  current_items->Clear();
  delete current_items;
} 

void Console::ConsoleInit(int argc, int argv[], pfScene *env)
{
  if(env == NULL)
    {
      cvrnPrintf("No Scene!!\n  call with ConsoleInit(int, int[], pfScene*)\n");
      return;
    }

  else pfs_pointer = env;
  
  // Insert yourself in the user list first
  // You are always head->next
  /////////////////////////////////////////
  current_users->Insert(g_limAvatarManager->selfAvatar);

  //register tether callbacks for message manager
  ////////////////////////////////////////
  g_ConsoleMessageMgr->registerMessage(consoleGlobal::_TETHER_MSG,
				       consoleGlobal::_USER_RELIABLE,
				       csTetherCB, 
                                       crTetherCB,
				       NULL, NULL);

  g_ConsoleMessageMgr->registerMessage(consoleGlobal::_TCONFIRM_MSG,
				       consoleGlobal::_USER_RELIABLE,
				       cs_tConfirmCB, 
                                       cr_tConfirmCB,
				       NULL, NULL);

  g_ConsoleMessageMgr->registerMessage(consoleGlobal::_TDETACH_MSG,
				       consoleGlobal::_USER_RELIABLE,
				       cs_tDetachCB, 
                                       cr_tDetachCB,
				       NULL, NULL);

  //register shareview callbacks for message manager
  //////////////////////////////////////////////////
  g_ConsoleMessageMgr->registerMessage(consoleGlobal::_SHARE_MSG,
				       consoleGlobal::_USER_RELIABLE,
				       csShareViewCB, 
                                       crShareViewCB,
				       NULL, NULL);

  g_ConsoleMessageMgr->registerMessage(consoleGlobal::_SCONFIRM_MSG,
				       consoleGlobal::_USER_RELIABLE,
				       cs_sConfirmCB, 
                                       cr_sConfirmCB,
				       NULL, NULL);

  g_ConsoleMessageMgr->registerMessage(consoleGlobal::_SDETACH_MSG,
				       consoleGlobal::_USER_RELIABLE,
				       cs_sDetachCB, 
                                       cr_sDetachCB,
				       NULL, NULL);

  
  for(int i=0; i<argc; i++)
    {
      switch(argv[i])
	{
	case egoFlatRadar:
	  My_fpRadar::init();
	  My_fpRadar *mfr = new My_fpRadar(egoFlatRadar);
	  cvrnPrintf("Returned from MFR constructor");
	  addConsoleItem((ConsoleItem *)mfr);
	  cvrnPrintf("ItemType as ConsoleItem = %d\n", 
		     ((ConsoleItem *)mfr)->getItemType());
	  
	  pfDCS *my_fpRadarDCS = new pfDCS();	  
	  mfr->setRot(0.0f, 90.0f, 0.0f);
	  my_fpRadarDCS->addChild(mfr);
	  env->addChild(my_fpRadarDCS);	
	  cvrnPrintf("flat radar added. item type is: %d\n", 
		     mfr->getItemType());
	  break;

	case egoRadar:
	  My_tsRadar::init();
	  My_tsRadar *mtr = new My_tsRadar(egoRadar);
	  addConsoleItem((ConsoleItem *)mtr);
	  cvrnPrintf("ItemType as ConsoleItem = %d\n", 
		     ((ConsoleItem *)mtr)->getItemType());
	  
	  pfDCS *my_tsRadarDCS = new pfDCS();	  
          mtr->setRot(0.0f, 90.0f, 0.0f);
	  my_tsRadarDCS->addChild(mtr);
	  env->addChild(my_tsRadarDCS);
	  cvrnPrintf("default radar added. item type is: %d\n", 
		     mtr->getItemType());
	  break;

	case partList:
	  PartList::init();
	  PartList *partList = new PartList();
	  addConsoleItem(partList);	  
	  
	  pfDCS *partListDCS = new pfDCS();	  	  
	  partListDCS->addChild(partList);
	  env->addChild(partListDCS);	  
	  break;

	case status_panel:
	  break;

	default:
	  break;
	}
    }
  
  reportList();
  return;
}

/*********************************************************************/
/** BEGINNING OF ADD & REMOVE CONSOLE ITEM FUNCTIONS FOR CONSOLE *****/
/** vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv *************/
/*********************************************************************/
  
void Console::addConsoleItem(const ConsoleItem * it)
{
  current_items->Append(it);
  return; 
}

ConsoleItem Console::removeConsoleItem(const ConsoleItem * it)
{
  current_items->SetFirst();
  current_items->Find(it);
  ConsoleItem temp = current_items->Remove();
  return temp;
}

/*********************************************************************/
/** BEGINNING OF ADD & REMOVE USER FUNCTIONS FOR CONSOLE *************/
/** vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv *************/
/*********************************************************************/


// This calls the addUser method of all console items on the console
// and appends *new_av to the list of current users tracked by console
void Console::addUser(CAVERNst_baseAvatar_c *new_user)
{
  current_users->Append(new_user);
  cvrnPrintf("user list = %d\n", current_users->GetLength());
  current_items->SetFirst();
  while(current_items->InList())
    {
      cvrnPrintf("NEXT ITEM IS: %d\n",
		 current_items->curr->next->object->getItemType());
      if(current_items->curr->next->object->isOfType(My_tsRadar::getClassType()))
	{
	  cvrnPrintf("TYPE is MFR\n");
	}
      else if(current_items->curr->next->object->isOfType(My_tsRadar::getClassType()))
	{
	  cvrnPrintf("TYPE is MR\n");
	}
      current_items->curr->next->object->addUser(new_user);
      current_items->Next();
    }
  return; 
}

// This calls the RemoveUser method of all console items on the console
void Console::removeUser(CAVERNst_baseAvatar_c *old_user)
{
  current_users->Find(old_user);
  CAVERNst_baseAvatar_c cu_temp = current_users->Remove();
  current_items->SetFirst();
  while(!current_items->IsEmpty() && current_items->InList())
    {
      ItemType it_temp =(current_items->curr->next->object->getItemType());
      if(it_temp==egoRadar || it_temp==egoFlatRadar || it_temp==partList)
	{
	  current_items->curr->next->object->removeUser(old_user);
	}
      current_items->Next();
    }
  return; 
} 

/*********************************************************************/
/** BEGINNING OF SHOW & HIDE FUNCTIONS FOR CONSOLE *******************/
/** vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv *************/
/*********************************************************************/

void Console::show(const char * it)
{
  current_items->find(it);
  cvrnPrintf("NEXT ITEM:  %s; shown = %d\n", 
		 current_items->curr->next->object->getName(),current_items->curr->next->object->getViewState());
  if(current_items->curr->next->object->getViewState() == 0)
    {
      cvrnPrintf("NEXT ITEM TO SHOW IS:  %s\n", 
		 current_items->curr->next->object->getName());
      current_items->curr->next->object->setTravMask(PFTRAV_DRAW, 
						     0xffffffff, 
						     PFTRAV_SELF | PFTRAV_DESCEND,
						     PF_SET);
      current_items->curr->next->object->setViewState(1);
    }
  else cvrnPrintf("ERROR: item already shown!\n");
  return;
}

void Console::hide(const char * it)
{
  current_items->find(it);
  if(current_items->curr->next->object->getViewState() == 1)
    {
      cvrnPrintf("NEXT ITEM HIDE IS:  %s\n", 
		 current_items->curr->next->object->getName());
      current_items->curr->next->object->setTravMask(PFTRAV_DRAW,
						     0x0, 
						     PFTRAV_SELF | PFTRAV_DESCEND,
						     PF_SET);
      current_items->curr->next->object->setViewState(0);
    }
  else cvrnPrintf("ERROR: item not shown!\n");
      return;
}

/*******************************************************************/
/** BEGIN SHAREVIEW FUNCTIONS **************************************/
/*******************************************************************/

/*******************************************/
/** CONSOLE CLASS FUNCTIONS ****************/
/** vvvvvvvvvvvvvvvvvvvvvvv ****************/
/*******************************************/

void Console::shareSend(int which, const char *u_name)
{
  /*
  // check for a redundancy
  /////////////////////////
  if (!strcmp(((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->shareWith, u_name))
    {
      cvrnPrintf("Already sharing with %s\n", u_name);
      return;
    }
    */

  // look for the user in your user list
  //////////////////////////////////////
  if(!current_users->IsEmpty() && current_users->find(u_name))
    {
      cvrnPrintf("WHICH = %d\n", which);
      switch(which)
	{
	case 9:
	  // send Request
	  cvrnPrintf("Switched to _SHARE_MSG...\n");
	  g_ConsoleMessageMgr->clientSend(consoleGlobal::_SHARE_MSG,
					  ((CAVERNst_baseAvatar_c *)(current_users->curr->next)->object));
	  break;
	case 6:
	  // send Request
	  cvrnPrintf("Switched to _DETACH_MSG...\n");
	  g_ConsoleMessageMgr->clientSend(consoleGlobal::_SDETACH_MSG,
					  ((CAVERNst_baseAvatar_c *)(current_users->curr->next)->object));
	  break;
	}
    }

  return;

}

void Console::shareLinkTo(const char *u_name)
{
  /*
  // check for a redundancy
  /////////////////////////
  if (!strcmp(((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->sharedWith, u_name))
    {
      cvrnPrintf("Already sharing view with: %s\n", u_name);
      return;
    }
    */
	
  CAVERNst_baseAvatar_c *theAvatar;     // pointer to an avatar in the avatar DB
  
  // Update your information in the DB.
  // This is done indirectly through current_users.
  // Don't forget to use the locks!!
  /////////////////////////////////////////////////
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->lock();
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->activeView = 1;
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->sharedWith = (char *)u_name;
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->unlock();
  
  
  cvrnPrintf("You are about to share views with:  %s\n", 
	     ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->sharedWith);
  
  theAvatar = (CAVERNst_baseAvatar_c*)current_users->curr->next->object;
  
  view = new ShareView(theAvatar);
  
  pfs_pointer->addChild(view);
  
  //cvrnPrintf("Tether Request rejected by %s\n", u_name);
  
  //else printf("USER %s NOT FOUND!!\n", u_name);
  return;
}

void Console::shareDetach(const char *u_name)
{
  cvrnPrintf("Attempting to Detach from %s...\n", u_name);
  if(strcmp(((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->sharedWith, (char *)u_name))
    {
      cvrnPrintf("You are not sharing views with %s!\n", u_name);
      cvrnPrintf("sharedWith = %s.\n",((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->sharedWith); 
      return;
    }     

  cvrnPrintf("Detaching View from %s.\n", u_name);
  //pfDelete((pfDCS *)pfs_pointer->removeChild(tether));

  //newTetherDCS->removeChild(geode);
  //geode->removeGSet(gSetRay);
  //pfDelete(gSetRay);
  //pfDelete(geode);
  
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->lock();
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->sharedWith = NULL;
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->activeView = 0;
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->unlock();

  
  return;
}


/*******************************************/
/** GLOBAL CALLBACK FUNCTIONS **************/
/** vvvvvvvvvvvvvvvvvvvvvvvvv **************/
/*******************************************/

void csShareViewCB(CAVERNst_baseAvatar_c * av,
                              CAVERNplus_datapack_c * datapack)
{
  cvrnPrintf("Sending ShareView Request...\n");
  datapack->packLong(av->avatarID);
  datapack->packLong(g_limAvatarManager->selfAvatar->avatarID );
}

void crShareViewCB(CAVERNplus_datapack_c * datapack)
{
  long recipID, avid;
  recipID = datapack->unpackLong();
  cvrnPrintf("Recipient ID for crShareViewCB = %d\n", recipID);
  if(recipID ==  g_limAvatarManager->selfAvatar->avatarID)
    {
      avid = datapack->unpackLong();
      cvrnPrintf("Received ShareView Request from %d\n", avid);
      if(current_users->find(avid))
	{
	  cvrnPrintf("Opening confirmation channel...\n");
	  g_ConsoleMessageMgr->clientSend(consoleGlobal::_SCONFIRM_MSG,
					  ((CAVERNst_baseAvatar_c *)(current_users->curr->next)->object));
	}
    }
      
  return;
}

void cs_sConfirmCB(CAVERNst_baseAvatar_c * av,
                              CAVERNplus_datapack_c * datapack)
{
  cvrnPrintf("Sending ShareView Confirmation to %d...\n", av->avatarID);
  datapack->packInt(1);
  datapack->packLong(g_limAvatarManager->selfAvatar->avatarID);
  datapack->packLong(av->avatarID);

  return;
}

void cr_sConfirmCB(CAVERNplus_datapack_c * datapack)
{
  long recipID, avid;
  int answer = 0;
  recipID = datapack->unpackLong();
  cvrnPrintf("Recipient ID for cr_sConfirmCB = %d\n", recipID);
  if(recipID == g_limAvatarManager->selfAvatar->avatarID)
    {
      avid = datapack->unpackLong();
      cvrnPrintf("Received ShareView Confirmation from %d\n", avid);
      if(current_users->find(avid))
	{
	  answer = datapack->unpackInt();
	  cvrnPrintf("answer = %d\n", answer);
	  if(answer == 1)
	    {
	      cvrnPrintf("Answer was YES...\n");
	      console->shareLinkTo(((CAVERNst_baseAvatar_c *)(current_users->curr->next)->object)->getName());
	    }
	}
    }
  return;
}

void cs_sDetachCB(CAVERNst_baseAvatar_c * av,
                              CAVERNplus_datapack_c * datapack)
{
  cvrnPrintf("Sending ShareView Detach.\n");
  datapack->packLong(av->avatarID);
  datapack->packLong(g_limAvatarManager->selfAvatar->avatarID );
}

void cr_sDetachCB(CAVERNplus_datapack_c * datapack)
{
  long myid, avid;
  myid = datapack->unpackLong();
  if(myid ==  g_limAvatarManager->selfAvatar->avatarID)
    {
      avid = datapack->unpackLong();
      cvrnPrintf("Received ShareView Detach Message from %d\n", avid);
      if(current_users->find(avid))
	{
	  console->shareDetach(((CAVERNst_baseAvatar_c *)(current_users->curr->next)->object)->getName());
	}
    }
      
  return;
}

/*******************************************************************/
/** END SHAREVIEW FUNCTIONS ****************************************/
/*******************************************************************/



/*******************************************************************/
/** BEGIN TETHER FUNCTIONS *****************************************/
/*******************************************************************/

/*******************************************/
/** CONSOLE CLASS FUNCTIONS ****************/
/** vvvvvvvvvvvvvvvvvvvvvvv ****************/
/*******************************************/

void Console::tetherSend(int which, const char *u_name)
{
  /*
  // check for a redundancy
  /////////////////////////
  if (!strcmp(((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->tetheredTo, u_name))
    {
      cvrnPrintf("Already tethered to %s\n", u_name);
      return;
    }
    */

  // look for the user in your user list
  //////////////////////////////////////
  if(!current_users->IsEmpty() && current_users->find(u_name))
    {
      cvrnPrintf("WHICH = %d\n", which);
      switch(which)
	{
	case 5:
	  // send Request
	  cvrnPrintf("Switched to _TETHER_MSG...\n");
	  g_ConsoleMessageMgr->clientSend(consoleGlobal::_TETHER_MSG,
					  ((CAVERNst_baseAvatar_c *)(current_users->curr->next)->object));
	  break;
	case 6:
	  // send Request
	  cvrnPrintf("Switched to _DETACH_MSG...\n");
	  g_ConsoleMessageMgr->clientSend(consoleGlobal::_TDETACH_MSG,
					  ((CAVERNst_baseAvatar_c *)(current_users->curr->next)->object));
	  break;
	}
    }

  return;

}

void Console::tetherLinkTo(const char *u_name)
{
  /*
  // check for a redundancy
  /////////////////////////
  if (!strcmp(((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->tetheredTo, u_name))
    {
      cvrnPrintf("Already tethered to %s\n", u_name);
      return;
    }
    */
	
  CAVERNst_baseAvatar_c *theAvatar;     // pointer to an avatar in the avatar DB
  
  // Update your information in the DB.
  // This is done indirectly through current_users.
  // Don't forget to use the locks!!
  /////////////////////////////////////////////////
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->lock();
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->activeTether = 1;
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->tetheredTo = (char *)u_name;
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->numTethers++;
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->unlock();
  
  
  cvrnPrintf("You are now being tethered to:  %s\n", 
	     ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->tetheredTo);
  
  theAvatar = (CAVERNst_baseAvatar_c*)current_users->curr->next->object;
  
  tether = new Tether(theAvatar);
  
  pfs_pointer->addChild(tether);
  
  //cvrnPrintf("Tether Request rejected by %s\n", u_name);
  
  //else printf("USER %s NOT FOUND!!\n", u_name);
  return;
}

void Console::tetherDetach(const char *u_name)
{
  cvrnPrintf("Attempting to Detach from %s...\n", u_name);
  if(strcmp(((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->tetheredTo, (char *)u_name))
    {
      cvrnPrintf("You are not tethered to %s!\n", u_name);
      cvrnPrintf("tetherTo = %s.\n",((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->tetheredTo); 
      return;
    }     

  cvrnPrintf("Detaching Tether from %s.\n", u_name);
  pfDelete((pfDCS *)pfs_pointer->removeChild(tether));

  //newTetherDCS->removeChild(geode);
  //geode->removeGSet(gSetRay);
  //pfDelete(gSetRay);
  //pfDelete(geode);
  
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->lock();
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->tetheredTo = NULL;
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->activeTether = 0;
  ((CAVERNst_baseAvatar_c *)(current_users->head->next->object))->unlock();

  
  return;
}


/*******************************************/
/** GLOBAL CALLBACK FUNCTIONS **************/
/** vvvvvvvvvvvvvvvvvvvvvvvvv **************/
/*******************************************/

void csTetherCB(CAVERNst_baseAvatar_c * av,
                              CAVERNplus_datapack_c * datapack)
{
  cvrnPrintf("Sending Tether...\n");
  datapack->packLong(av->avatarID);
  datapack->packLong(g_limAvatarManager->selfAvatar->avatarID );
}

void crTetherCB(CAVERNplus_datapack_c * datapack)
{
  long recipID, avid;
  recipID = datapack->unpackLong();
  cvrnPrintf("Recipient ID for crTetherCB = %d\n", recipID);
  if(recipID ==  g_limAvatarManager->selfAvatar->avatarID)
    {
      avid = datapack->unpackLong();
      cvrnPrintf("Received Tether Request from %d\n", avid);
      if(current_users->find(avid))
	{
	  /*
	  cvrnPrintf("Opening confirmation channel...\n");
	  g_ConsoleMessageMgr->clientSend(consoleGlobal::_TCONFIRM_MSG,
					  ((CAVERNst_baseAvatar_c *)(current_users->curr->next)->object));
					  */
	   console->tetherLinkTo(((CAVERNst_baseAvatar_c *)(current_users->curr->next)->object)->getName());
	}
    }
      
  return;
}

void cs_tConfirmCB(CAVERNst_baseAvatar_c * av,
                              CAVERNplus_datapack_c * datapack)
{
  cvrnPrintf("Sending Confirmation to %d...\n", av->avatarID);
  datapack->packInt(1);
  datapack->packLong(g_limAvatarManager->selfAvatar->avatarID);
  datapack->packLong(av->avatarID);

  return;
}

void cr_tConfirmCB(CAVERNplus_datapack_c * datapack)
{
  long recipID, avid;
  int answer = 0;
  recipID = datapack->unpackLong();
  cvrnPrintf("Recipient ID for cr_tConfirmCB = %d\n", recipID);
  if(recipID == g_limAvatarManager->selfAvatar->avatarID)
    {
      avid = datapack->unpackLong();
      cvrnPrintf("Received Tether Confirmation from %d\n", avid);
      if(current_users->find(avid))
	{
	  answer = datapack->unpackInt();
	  cvrnPrintf("answer = %d\n", answer);
	  if(answer == 1)
	    {
	      cvrnPrintf("Answer was YES...\n");
	      console->tetherLinkTo(((CAVERNst_baseAvatar_c *)(current_users->curr->next)->object)->getName());
	    }
	}
    }
  return;
}

void cs_tDetachCB(CAVERNst_baseAvatar_c * av,
                              CAVERNplus_datapack_c * datapack)
{
  cvrnPrintf("Sending Detach.\n");
  datapack->packLong(av->avatarID);
  datapack->packLong(g_limAvatarManager->selfAvatar->avatarID );
}

void cr_tDetachCB(CAVERNplus_datapack_c * datapack)
{
  long myid, avid;
  myid = datapack->unpackLong();
  if(myid ==  g_limAvatarManager->selfAvatar->avatarID)
    {
      avid = datapack->unpackLong();
      cvrnPrintf("Received Detach Message from %d\n", avid);
      if(current_users->find(avid))
	{
	  console->tetherDetach(((CAVERNst_baseAvatar_c *)(current_users->curr->next)->object)->getName());
	}
    }
      
  return;
}

/*******************************************************************/
/** END TETHER FUNCTIONS *******************************************/
/*******************************************************************/

void Console::reportList()
{
  if(current_items->IsEmpty())
    {
      cvrnPrintf("Item List is EMPTY!\n");
      return;
    }
  cvrnPrintf("***********\n");
  cvrnPrintf("LIST REPORT\n");
  cvrnPrintf("***********\n");
  current_items->SetFirst();
  for(int i=0; i<current_items->GetLength(); i++)
    {
      cvrnPrintf("item %d = %d\n", i, 
		 current_items->curr->next->object->getItemType());
      current_items->Next();
    }

  return;
}


// 
//int Console::Update_ONLY(char *item, int property, char *user)
//{ 
//if(current_items->Find(item))
//{
//current_items->curr->next->object->Update_ONLY(user, property);
//return 1;
//}
//return 0; 
//}
