/*
 * LIMBO
 * 4/3/98
 * (C) 1998 by Jason Leigh
 * Univ Illinois at Chicago

 This program delivers a 3D model to a LIMBO client or its server.
 
 */

#include "CAVERN.h++"
#include "lim_appName.h++"

main(int argc, char** argv)
{

	if (argc != 5) {
		fprintf(stderr,"\n%s world_name remoteIP <local_object_file | NONE> id\n\n",
			argv[0]);
		fprintf(stderr,"world_name        : the name of the virtual world (default is limbo)\n");
		fprintf(stderr,"remoteIP          : remote address to send object to.\n");
		fprintf(stderr,"local_object_file : local 3D object file.\n");
		fprintf(stderr,"                    NONE is to remove the object.\n");
		fprintf(stderr,"id                : the object id to use (a number between 0 and 63)\n");
		exit(1);
	}

	CAVERN_irb_c::status_t irbStatus;
	CAVERN_irbId_c remoteIRBId;
	CAVERN_irbChannel_c::status_t channelRetStatus;
	CAVERN_irbKey_c::status_t keyStatus;

	CAVERN_initAttrib_c initAttr;
	initAttr.setPort(9761);

	/// Init CAVERN.
	CAVERN_irb_c *personalIRB = CAVERNInit(&argc, &argv, &initAttr);
	if (!personalIRB) exit(1);

	/// Create a channel over which communication will occur.
	CAVERN_irbChannel_c *aChannel = personalIRB->createChannel();

	if (aChannel == NULL) {
		cvrnPrintf("Unable to open channel\n");
		exit(1);
	}

	/// Open the channel to a remote IRB specified on the command line.
	remoteIRBId.setAddress(argv[2]);
	aChannel->open(&remoteIRBId,NULL,CAVERN_irbChannel_c::RELIABLE, &channelRetStatus);

	if (getenv("LIMBO_REMOTE_PORT"))
		remoteIRBId.setPort(atoi(getenv("LIMBO_REMOTE_PORT")));

	/// Define the local key (called LOCAL_A_KEY)
	CAVERN_irbKeyId_c aKeyId, remote_aKeyId;
	aKeyId.setName("DBDELIVER_KEY");

	char str[256];
	sprintf(str,"Mo%s",argv[4]);
	remote_aKeyId.setName(str);
	remote_aKeyId.setPath(argv[1]);


	CAVERN_irbKey_c *aKey = personalIRB->define(&aKeyId, NULL, &irbStatus);

	/// Create a link to link the local key with the remote key.
	CAVERN_linkAttrib_c linkAttribute;
	linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::NONE);
	linkAttribute.setSubsequentSynchType(CAVERN_linkAttrib_c::FORCE_LOCAL_TO_REMOTE);
	CAVERN_irbLink_c *aLink = aChannel->link(aKey, &remote_aKeyId, &linkAttribute);

	if (aLink == NULL) {
		cvrnPrintf("delimbo: Unable to create link\n");
		exit(1);
	}

	CAVERN_irbLink_c::status_t linkStatus;

	int size = 0;

	// If none is specified then a null object is distributed to remove objects from
	// the LIMBO world.
	int len;
	if (strcmp(argv[3],"NONE")) {
		len = strlen(argv[3])+1;
		aKey->import(argv[3], &size, argv[3], &len, &keyStatus, CAVERN_irbKey_c::BLOCKING);
		if (keyStatus == CAVERN_irbKey_c::OK) {
			aLink->commitRemote(&linkStatus);
			cvrnPrintf("Transfer done. %d bytes.\n", size);
		} else {
			cvrnPrintf("Transfer failed.\n");
		}
	} else {
		aKey->put(NULL, &size, &keyStatus);
		aLink->commitRemote(&linkStatus);
	}


	delete aLink;
	delete aChannel;
	delete personalIRB;
}
