/*
 * LIMBO
 * 4/3/98
 * (C) 1998 by Jason Leigh
 * Univ Illinois at Chicago
 *
 */

#include "ra.h++"
#include "lim_global.h++"
#include "lim_init.h++"
#include "lim_perf.h++"

CAVERN_irbChannel_c *g_limAudioChannel = NULL;
CAVERN_irbKey_c *g_limAudioKey=NULL;
CAVERN_irbLink_c *g_limAudioLink= NULL;

void createAudioChannel();
void makeChannelAndAudioLink(char *remoteHost);
void newAudioCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData);

// Initialize audio
void limAudioInit()
{
	CAVERN_irb_c::status_t irbStatus;
	CAVERN_irbKeyId_c audioKeyId;

	audioKeyId.setName("AUDIO_KEY");
	audioKeyId.setPath(g_worldName);

	g_limAudioKey = g_limPIRB->define(&audioKeyId, NULL, &irbStatus);
	g_limAudioKey->trigger(newAudioCB, (void *) NULL);

	if (g_limRemoteConnect) {
		// Open the channel to a remote IRB specified on the command line.
		createAudioChannel();
		makeChannelAndAudioLink(g_worldServer);
	}

}

void createAudioChannel()
{
	g_limAudioChannel = g_limPIRB->createChannel();
}


// Make a channel and a link to link the AUDIO_KEY
void makeChannelAndAudioLink(char *remoteHost)
{
	CAVERN_irbKeyId_c audioKeyId;
	audioKeyId.setName("AUDIO_KEY");
	audioKeyId.setPath(g_worldName);

	CAVERN_irbChannel_c::status_t channelRetStatus;

	CAVERN_irbId_c remoteIRBId;
	remoteIRBId.setAddress(remoteHost);
	remoteIRBId.setPort(g_limRemotePort);

	do {
		cvrnPrintf("Audio Channel: Attempting to Connect...\n");
		g_limAudioChannel->open(&remoteIRBId,NULL,CAVERN_irbChannel_c::RELIABLE,
		       &channelRetStatus);
		sleep(1);
	} while(channelRetStatus != CAVERN_irbChannel_c::OK);

	cvrnPrintf("Audio Channel: Connected\n");

	if (g_limAudioChannel) {

		if (g_limAudioLink) delete g_limAudioLink;

		CAVERN_linkAttrib_c linkAttribute;

		linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::NONE);
		g_limAudioLink = g_limAudioChannel->link(g_limAudioKey, &audioKeyId,&linkAttribute);
		if (g_limAudioLink == NULL) {
			cvrnPrintf("Audio Channel: The link didn't work!\n");
			fflush(stdout);
		}
	}
}

// This handles the recording of audio for the intercom.
// Basically the user presses the right wand button and says something
// into the microphone. That audio is recorded and sent to all the
// remote participants.
void limHandleRecording()
{
	static int recordState = 0;
	static audioRecorder *ro=NULL;
	int size;
	CAVERN_irbKey_c::status_t status;
	float wandPos[3];
	static char outfile[1024];

	// Hit the "speak button" and talk
	if (CAVEBUTTON3) {
		if (recordState == 0) {

			// See ra.c++ and ra.h++ for more info
			ro = new audioRecorder;
			sprintf(outfile,"%s.aiff",
				tempnam("/usr/tmp","limboaudio"));

			ro->init(outfile);
			recordState = 1;

		} else {
			// Draw a microphone
			CAVEGetPosition(CAVE_HEAD, wandPos);
			g_limMicDCS->setTrans(wandPos[0]+1, wandPos[1]+3, wandPos[2]-0.5);
			ro->record();
		}
	} else {
		// Stop recording when the button is released and send off audio.
		if (recordState) {
			g_limMicDCS->setTrans(0,-10,-2);

			ro->stop();

			g_limAudioKey->import(outfile,&size, &status);
			delete ro;
			unlink(outfile);
			recordState = 0;
		}
	}
}


// Callback for when a new audio arrives.
// When new audio arrives play it back.
void newAudioCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData)
{
	int size;

	CAVERN_irbKey_c::status_t status;
	thisKey->getSize(&size, &status);

	char outfile[128];

	sprintf(outfile,"%s.aiff",
		tempnam("/usr/tmp","limboaudio"));

	char command[128];

	if (size) {
		cvrnPrintf("AUDIO DATA ARRIVED. EXPORTING\n");
		thisKey->export(outfile, &status);
		cvrnPrintf("EXPORTING DONE\n");
		sprintf(command,"playaiff %s &",outfile);
		system(command);
	}
}

void playSound(char *file)
{
	char command[1024];
	char *lpath=getenv("LIMBO");

	if (lpath)
		sprintf(command,"playaiff %s/LIMBO_SOUNDS/%s &", lpath,file);
	else
		sprintf(command,"playaiff LIMBO_SOUNDS/%s &", file);

	system(command);

}
