/*
 * LIMBO
 * 4/3/98
 * (C) 1998 by Jason Leigh
 * Univ Illinois at Chicago
 *
 */

#include "lim_global.h++"
#include "lim_audio.h++"
#include "lim_model.h++"
#include "lim_perf.h++"
#include "lim_avatar.h++"
#include "lim_logcode.h++"
#include "lim_user.h++"
//#include "catch.h"

char* limVersion = "LIMBO (version 0.42 12/10/98)\n(C)1998 Jason Leigh, Andrew E. Johnson, Electronic Visualization Lab & NCSA\nUniv. Illinois at Chicago.";


char g_userName[256];
char *g_worldServer = NULL;
char g_worldName[256];
long g_limLocalPort = 10000;
long g_limRemotePort = 10000;

// CAVERN Personal IRB
CAVERN_irb_c *g_limPIRB = NULL;
void limCAVERNInit(int argc, char** argv);

// Flag to determine if LIMBO will create a remote connection.
int g_limRemoteConnect = 0;

void parseCommandLineOptions(int argc, char **argv)
{
	char *s;

	strcpy(g_userName,"Anonymous");
	strcpy(g_worldName,_LIM_APP_NAME);

	for (int i = 1; i < argc; i++) {
		s= argv[i];

		if (strlen(s) > 1 && !strncmp(s,"-n",2)) {
			i++;
			strcpy(g_userName,argv[i]);
		} else if (strlen(s) > 1 && !strncmp(s,"-s",2)) {
			i++;
			g_worldServer = argv[i];
			g_limRemoteConnect = 1;
		} else if (strlen(s) > 1 && !strncmp(s,"-w",2)) {
			i++;
			strcpy(g_worldName,argv[i]);
		} else if (strlen(s) > 1 && !strncmp(s,"-l",2)) {
			i++;
			g_limLocalPort = atol(argv[i]);
		} else if (strlen(s) > 1 && !strncmp(s,"-r",2)) {
			i++;
			g_limRemotePort = atol(argv[i]);
		} else if (strlen(s) > 1 && !strncmp(s,"-h",2)) {
			fprintf(stderr,"%s: usage: %s [-n your_name] [-w world_name] [-s server_address] [-l local_port] [-r remote_port] [-help]\n",argv[0], argv[0]);
			exit(1);
			
		}
	}
}


// Initialize limbo
void limInit(int argc, char** argv)
{
	printf("%s\n\n",limVersion);

	parseCommandLineOptions(argc, argv);

	printf("User Name   : %s\n", g_userName);
	printf("World Name  : %s\n", g_worldName);
	if (g_worldServer)
		printf("World Server: %s\n", g_worldServer); 
	else
		printf("World Server: SELF\n");


	// Initialize Performer
	limPerformerInit(argc, argv);

	// Add avatar body parts to the performer scene.
	limAddAvatarMainDCS();

	// Initialize CAVERN
	limCAVERNInit(argc, argv);

	// Initialize user data channels- do this last
	limUserDataInit();

	// Initialize audio intercom hook up
	limAudioInit();

	// Initialize model loading
	limModelInit();

	// Cycle through performer program once
	for(int i =0; i < 30; i++)
	limPerfOneCycle();

	// Initialize avatars
	limAvatarInit();
	
	//catchInit();
}

// Initialize CAVERN
void limCAVERNInit(int argc, char** argv)
{

	CAVERN_initAttrib_c initAttr;

	initPerformanceMonitoring(&initAttr);

	initAttr.setPort(g_limLocalPort);

	if (getenv("LIMBO_DB")) {
		initAttr.setDBName(getenv("LIMBO_DB"));
	}


	/// Startup CAVERN.
	g_limPIRB = CAVERNInit(&argc, &argv, &initAttr);
	if (!g_limPIRB) {
		exit(1);
	}
	
}

