/*
At some point in the near future these performance monitoring calls will be
usable.

For now they have been ifdefed out. Performance monitoring requires
CAVERN 2.2 or beyond.

Environment variables that can be used to control performance monitoring
include:
LIMBO_DO_PERF
LIMBO_PERF_HOST
LIMBO_PERF_ORG
LIMBO_PERF_LOCAL_LOG_RATE
LIMBO_PERF_REMOTE_LOG_RATE

*/

#include "CAVERN.h++"
#include <sys/types.h>
#include <unistd.h>


void initPerformanceMonitoring(CAVERN_initAttrib_c *initAttr) {
#ifdef _LIMBO_PERF_MONITORING
	char *perfHost, *perfOrg;
	perfHost = getenv("LIMBO_PERF_HOST");
	perfOrg = getenv("LIMBO_PERF_ORG");
	pid_t pid;
	pid = getpid();

	//	if (getenv("CAVERN_PORT"))
	//	initAttr->setPort(atoi(getenv("CAVERN_PORT")));


	struct tm *tim;
	time_t theClock;

	theClock = time(NULL);
	tim = localtime(&theClock);
	char timeStamp[256];

	sprintf(timeStamp,"%02d%02d%02d_%02d:%02d:%02d_%d.log",
		tim->tm_year,
		tim->tm_mon,
		tim->tm_mday,
		tim->tm_hour,
		tim->tm_min,
		tim->tm_sec,
		pid);

	if (getenv("LIMBO_DO_PERF")) {

		printf("PERF: PERFORMANCE MONITORING ENABLED\n");
		if (perfHost) {
			printf("PERF: REMOTE MONITORING ENABLED\n");

			initAttr->setLogRemoteHost(perfHost);
			initAttr->setLogRemotePort(14830);
		}
		if (perfOrg)
			initAttr->setLogOrganization(perfOrg);

		if (!getenv("LIMBO_NO_LOCAL_LOG"))
		initAttr->setLogLocalFile(timeStamp);
	}
#endif
}

void monitorChannel(CAVERN_irbChannel_c *aChannel,
		    char *label, int llRate, int rlRate)
{
#ifdef _LIMBO_PERF_MONITORING

	if (getenv("LIMBO_DO_PERF")) {
		aChannel->setLabel(label);
		aChannel->setRemoteLabel(label);

		int logRate = llRate;
		int remoteLogRate = rlRate;

		char *llr, *rlr;
		llr = getenv("LIMBO_PERF_LOCAL_LOG_RATE");
		rlr = getenv("LIMBO_PERF_REMOTE_LOG_RATE");

		if (llr) logRate = atoi(llr);
		if (rlr) remoteLogRate = atoi(rlr);

		aChannel->setNetworkLogRate(logRate);
		aChannel->setRemoteNetworkLogRate(remoteLogRate);

		printf("LOCAL LOG RATE: %d REMOTE LOG RATE: %d\n", logRate, remoteLogRate);
	}
#endif

}


