/*
 * LIMBO
 * 4/3/98
 * (C) 1998 by Jason Leigh
 * Univ Illinois at Chicago
 *
 */

#include "lim_global.h++"
#include "lim_perf.h++"
#include "lim_main.h++"
#include "lim_init.h++"
#include "lim_model.h++"
#include "lim_logcode.h++"
#include "lim_audio.h++"

// The model channel delivers all the 3D model data.
CAVERN_irbChannel_c *g_limModelChannel = NULL;

// The meta channel delivers all the position info about the model.
CAVERN_irbChannel_c *g_limModelMetaChannel = NULL;
CAVERN_irbLink_c *g_limModelLink[_LIM_MAX_MODELS];
CAVERN_irbLink_c *g_limModelMetaLink[_LIM_MAX_MODELS];

// Model keys hold the actual data of the model.
CAVERN_irbKey_c *g_limListOfModelDataKeys[_LIM_MAX_MODELS];

// Meta keys hold the position info about the model.
CAVERN_irbKey_c *g_limListOfModelMetaDataKeys[_LIM_MAX_MODELS];

// Performer DCS' for all the models.
pfDCS *g_listOfModelDCS[_LIM_MAX_MODELS];

// Flag to tell if a new model has been downloaded.
// Checked by limCheckForNewObjectsToLoad()
int g_replacementFlag[_LIM_MAX_MODELS];
char g_replacementFile[_LIM_MAX_MODELS][128];
#define _LIM_MODEL_REPLACEMENT_NEEDED 1
#define _LIM_MODEL_REPLACEMENT_DONE 0
#define _LIM_MODEL_REMOVAL -1

/************************************************************************/
void newModelCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData);

void newMetaDataCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData);

void openModelChannels(char *remoteHost);
void makeModelLinks(int index);

/************************************************************************/

void limMarkModelForRemoval(int index)
{
	int size = 0;
	CAVERN_irbKey_c::status_t status;

	CAVERN_irbKey_c *aKey = g_limListOfModelDataKeys[index];
	aKey->put(NULL, &size, &status);
	aKey->commit(&status);

	g_replacementFlag[index] = _LIM_MODEL_REMOVAL;
	playSound("flash.aif");

}

// Initialize model keys, links and channels.
void limModelInit()
{
	for(int i = 0; i < _LIM_MAX_MODELS; i++) g_limListOfModelMetaDataKeys[i] = NULL;

	CAVERN_irbKeyId_c modelKeyId, modelMetaKeyId;
	CAVERN_irb_c::status_t irbStatus;

	// If a remote connection is expected create the channels for it.
	if (g_limRemoteConnect) {
		g_limModelChannel = g_limPIRB->createChannel();
		g_limModelMetaChannel = g_limPIRB->createChannel();
		openModelChannels(g_worldServer);
	}

	// Create a key and a link for each model data and model's meta data.
	for (i = 0; i < _LIM_MAX_MODELS; i++) {

		g_replacementFlag[i] = _LIM_MODEL_REPLACEMENT_DONE;

		// Model data key is labelled as Mo followed by a number between 0 and _LIM_MAX_MODELS-1
		char str[1024];
		sprintf(str,"Mo%d",i);
		modelKeyId.setPath(g_worldName);
		modelKeyId.setName(str);
		g_limListOfModelDataKeys[i] = g_limPIRB->define(&modelKeyId, NULL, &irbStatus);

		// If there is data already in the key then load it as a model.
		int size;
		CAVERN_irbKey_c::status_t status;
		g_limListOfModelDataKeys[i]->getSize(&size, &status);

		if (size) {
			//			cvrnPrintf("SIZE: %d\n",size);
			// Call the callback once to load any models that are in the local cache.
			newModelCB(CAVERN_irbKey_c::NEW_DATA_ARRIVED, g_limListOfModelDataKeys[i],
				   (void *) &i);
		}
			   
		// Model meta data key is labelled as Me followed by a number between 0 and _LIM_MAX_MODELS-1
		sprintf(str,"Me%d",i);
		modelMetaKeyId.setName(str);
		modelMetaKeyId.setPath(g_worldName);

		g_limListOfModelMetaDataKeys[i] = g_limPIRB->define(&modelMetaKeyId, NULL, &irbStatus);

		// Call the callback once to load any model meta data that are in the local cache.
		newMetaDataCB(CAVERN_irbKey_c::NEW_DATA_ARRIVED, g_limListOfModelMetaDataKeys[i],
			      (void *) g_listOfModelDCS[i]);

		// Prepare to receive new data from remote sites.
		g_limListOfModelDataKeys[i]->trigger(newModelCB, (void *) new int(i));
		g_limListOfModelMetaDataKeys[i]->trigger(newMetaDataCB, (void *) g_listOfModelDCS[i]);

		// Now form each link between the local and remote keys.
		if (g_limRemoteConnect) {
			makeModelLinks(i);
		}

	}


}

// Save the state of the models' meta data in CAVERN.
void limSaveModelMetaDataStates()
{
	CAVERN_irbKey_c::status_t status;
	CAVERN_irbLink_c::status_t statusL;

	cvrnPrintf("LIMBO: Shutting down- please wait.\n");
	for (int i = 0; i < _LIM_MAX_MODELS; i++) {
		cvrnPrintf(".");
		g_limListOfModelMetaDataKeys[i]->commit(&status);
		if (g_limRemoteConnect)
			g_limModelMetaLink[i]->commitRemote(&statusL);
	}
	cvrnPrintf("LIMBO: GOODBYE\n");
}

// Open channels to remote site.
void openModelChannels(char *remoteHost)
{
	CAVERN_irbChannel_c::status_t channelRetStatus;

	CAVERN_irbId_c remoteIRBId;
	remoteIRBId.setAddress(remoteHost);

	remoteIRBId.setPort(g_limRemotePort);
	
	do {
		cvrnPrintf("LIMBO: Model Channel: Attempting to Connect...\n");
		g_limModelChannel->open(&remoteIRBId,NULL,CAVERN_irbChannel_c::RELIABLE,
		       &channelRetStatus);
		sleep(1);
	} while(channelRetStatus != CAVERN_irbChannel_c::OK);
	cvrnPrintf("LIMBO: Model Channel: Connected\n");
	monitorChannel(g_limModelChannel, "limbo_MODEL",30,30);

	do {
		cvrnPrintf("LIMBO: Model Meta Channel: Attempting to Connect...\n");
		g_limModelMetaChannel->open(&remoteIRBId,NULL,CAVERN_irbChannel_c::UNRELIABLE,
		       &channelRetStatus);
		sleep(1);
	} while(channelRetStatus != CAVERN_irbChannel_c::OK);
	cvrnPrintf("LIMBO: Model Meta Channel: Connected\n");
	monitorChannel(g_limModelMetaChannel, "limbo_META",10,10);
}

// Make a link to link the model and model meta keys
void makeModelLinks(int index)
{
	CAVERN_irbKeyId_c modelKeyId;
	CAVERN_irbKeyId_c modelMetaKeyId;

	char str[1024];
	sprintf(str,"Mo%d",index);
	modelKeyId.setName(str);
	modelKeyId.setPath(g_worldName);


	sprintf(str,"Me%d",index);
	modelMetaKeyId.setPath(g_worldName);
	modelMetaKeyId.setName(str);

	if (g_limModelChannel) {

		// Create a link to link the local key named MODEL_KEY with the remote key
		// also named MODEL_KEY.
		CAVERN_linkAttrib_c linkAttribute;
		g_limModelLink[index] = g_limModelChannel->link(g_limListOfModelDataKeys[index], &modelKeyId,&linkAttribute);
		if (g_limModelLink[index] == NULL) {
			cvrnPrintf("LIMBO: Model Channel: The link didn't work!\n");
		}
	}

	if (g_limModelMetaChannel) {

		// Create a link to link the local key named MODEL_KEY with the remote key
		// also named MODEL_KEY. If you are connecting to a remote site then force
		// synch from the remote to the local.
		CAVERN_linkAttrib_c linkAttribute;
		linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::FORCE_REMOTE_TO_LOCAL);

		g_limModelMetaLink[index] = g_limModelChannel->link(g_limListOfModelMetaDataKeys[index], &modelMetaKeyId,&linkAttribute);
		if (g_limModelMetaLink[index] == NULL) {
			cvrnPrintf("LIMBO: Model Channel: The model meta link didn't work!\n");
		}
	}

}


// Replace any '/' in the meta data of the model key with an '_'
void cleanMeta(char *meta, int size)
{
	for (int i = 0; i < size; i++)
		if (meta[i] == '/') meta[i]='_';
}

// Callback for when a new model arrives.
void newModelCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData)
{

	int *index = (int *) userData;

	int size;
	CAVERN_irbKey_c::status_t status;
	thisKey->getSize(&size, &status);
	if (size) {

		char outfile[1024];
		char meta[128];
		int metaSize = 128;
		thisKey->getMeta(meta, &metaSize, &status);

		cleanMeta(meta, metaSize);

		sprintf(outfile,"%s%d%s",
			tempnam("/usr/tmp", "limbo"),
			*index, meta);

		thisKey->export(outfile, &status);

		thisKey->commit(&status);

		// Ordinarily I would do a performer load right
		// after the export but it seems something about Performer
		// really barfs. So it works on an O2 but not an Onyx2.
		// So instead I signal that the model has been exported
		// and let the main loop do the loading by calling
		// limCheckForNewObjectsToLoad().
		g_replacementFlag[*index] = _LIM_MODEL_REPLACEMENT_NEEDED;
		strcpy(g_replacementFile[*index],outfile);
	} else {
		// If null data arrives that means an attempt has been
		// made to remove the model.
		thisKey->commit(&status);
		g_replacementFlag[*index] = _LIM_MODEL_REMOVAL;
	}


}

// New incoming object position data arrives.
void newMetaDataCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData)
{
	float vec[16];
	int size;
	CAVERN_irbKey_c::status_t status;
	thisKey->getSize(&size, &status);

	/* FOR BACKWARDS COMPATIBILITY
	if (size == (sizeof(float) * 3)) {

		// Pull out the position data.
		thisKey->get((char*) vec, &size, &status);

		// Apply this to the dcs.
		
		pfDCS *aDCS = (pfDCS*) userData;
		aDCS->setTrans(vec[0], vec[1], vec[2]);
	}
	*/

	if (size == (sizeof(float) * 16)) {

		// Pull out the position data.
		thisKey->get((char*) vec, &size, &status);

		// Apply this to the dcs.
		
		pfDCS *aDCS = (pfDCS*) userData;
		pfMatrix newMat(vec[0],  vec[1],  vec[2],  vec[3], 
				vec[4],  vec[5],  vec[6],  vec[7], 
				vec[8],  vec[9],  vec[10], vec[11], 
				vec[12], vec[13], vec[14], vec[15]); 
		aDCS->setMat(newMat);
	}

}


// Check whether any of the models need to be reloaded.
void limCheckForNewObjectsToLoad(){

	for (int i = 0; i < _LIM_MAX_MODELS; i++) {
		if (g_replacementFlag[i] == _LIM_MODEL_REPLACEMENT_NEEDED) {

			cvrnPrintf("LIMBO: Performer Load: %d\n",i);

			limPerfReplaceObject(g_replacementFile[i], g_listOfModelDCS[i]);
			unlink(g_replacementFile[i]);

			g_replacementFlag[i] = _LIM_MODEL_REPLACEMENT_DONE;

			cvrnPrintf("LIMBO: Performer Done Loading.\n");

			playSound("loadsound.aif");
		} else if (g_replacementFlag[i] == _LIM_MODEL_REMOVAL) {
			limPerfRemoveObject(g_listOfModelDCS[i]);
			g_replacementFlag[i] = _LIM_MODEL_REPLACEMENT_DONE;

			// Reset the position to origin
			float vec[16];
			for (int j = 0; j < 16; j++) vec[j]=0;
			vec[0] = vec[5] = vec[10] = vec[15] = 1;

			int len = sizeof(float) * 16;
			CAVERN_irbKey_c::status_t status;
			CAVERN_irbKey_c *aKey;
			aKey = g_limListOfModelMetaDataKeys[i];
			aKey->put((char*) vec, &len, &status);
		}
	}
}

/* Use this function to add a 3D object to a vacant slot in the scene.
This function is not used in the basic version of LIMBO but it is
a handy function to have if your LIMBO client wants to dynamically
load in a new object file. Note that since no mutual exclusion is
guaranteed between sites it is possible for 2 clients to accidentally
replace each other's objects if both attempt to call this at the same
time.
*/
void limAddModelToVacantSlot(char *filename)
{
	int size, namelen;
	CAVERN_irbKey_c::status_t status;

	namelen = strlen(filename)+1;

	for (int i =0 ; i < _LIM_MAX_MODELS; i++) {
		g_limListOfModelDataKeys[i]->getSize(&size, &status);
		if (size == 0) {
			g_limListOfModelDataKeys[i]->import(filename,&size,filename,&namelen,&status);

			newModelCB(CAVERN_irbKey_c::NEW_DATA_ARRIVED, g_limListOfModelDataKeys[i],
				   (void *) &i);

			return;
		}
	}
	cvrnPrintf("LIMBO: Sorry: No room left to load model.\n");
}

void limMaterializeAsPersistentObject(pfNode *anode)
{
	char fpath[1024];

	sprintf(fpath,"%s.pfb",tempnam("/usr/tmp","limbo"));

	limExport(anode, fpath);
	limAddModelToVacantSlot(fpath);
	unlink(fpath);
}
