/*
 * LIMBO
 * 4/3/98
 * (C) 1998 by Jason Leigh
 * Univ Illinois at Chicago
 *
 * Navigation routines that include wall and floor collision detection.
 * This will allow you to climb objects.
 *
 */

#include "lim_perf.h++"
#include "lim_navigate.h++"

// Max cruise speed during navigation
#define _LIM_CRUISE_SPEED 1.0

// Max rotation speed during navigation
#define _LIM_TURN_SPEED 2

// Min walk height -10 feet
// This means you can only fall down to a level of -10 feet
#define _LIM_NAV_MIN_HEIGHT -10

// Current turn angle of the CAVE relative to the entire world
float g_limCurrentCaveTurn = 0;

// This turns off collision detection when flying towards a selected object.
int g_batmanMode = 0;

/************************************************************************/

float compute_nav_height()
{
	pfSegSet segset;
	float head[3];
	pfHit **downHits[32];
	int isect = NULL;
	static float fall = 0;
	float heightChange;
    

	segset.mode= PFTRAV_IS_PATH | PFTRAV_IS_PRIM | PFTRAV_IS_CULL_BACK;
	segset.userData	= NULL;   
	segset.activeMask   = 1;
	segset.isectMask    = 0x8; 
	segset.bound	= NULL;
	segset.discFunc	= NULL;

	/*********************************************/
	/* start the vector at my head pointing down */
	/*********************************************/

	pfCAVEGetPosition(CAVE_HEAD,head);
    
	segset.segs[0].dir.set(0.0f, 0.0f, -1.0f);
	segset.segs[0].length = 2000.0f;

	segset.segs[0].pos.set(head[0], head[1], head[2]);

	/**************************************/
	/* see if there is something below me */
	/**************************************/
    
	isect = g_limScene->isect(&segset, downHits);

	/**********************************/
	/* if there is something below me */
	/* how far below me is the ground */
	/**********************************/

	if (isect) {
		pfVec3 pnt, xpnt;
		pfMatrix xmat;
	
		(*downHits[0])->query(PFQHIT_POINT, pnt.vec);
		(*downHits[0])->query(PFQHIT_XFORM, (float*)xmat.mat);
	
		xpnt.xformPt(pnt, xmat);

		fall = 	xpnt[PF_Z];
		//	fprintf(stderr, "on something\n");	
	}
	else {
		fall = 0;
		//	fprintf(stderr, "floating\n");
	}
    

	/*************************************/
	/* interpolate the walking / falling */
	/*************************************/
    
#define _LIM_UP_DOWN_RATE 0.33f
#define _LIM_MAX_FALL_RATE -0.5f 
    
	heightChange = fall * _LIM_UP_DOWN_RATE;

	if ( heightChange < _LIM_MAX_FALL_RATE)
		heightChange = _LIM_MAX_FALL_RATE;

	CAVENavTranslate(0, 0, heightChange);
    
	return(heightChange);

}

/***********************************************************************/
/* This function is supposed to deal with keeping the user from        */
/* moving through solid objects while navigating with the joystick     */
/***********************************************************************/

int checkForWall(float xvec, float yvec, float zvec)
{
    
#define WALL_TEST_LENGTH 20.0f
#define WALL_HIT_RADIUS   3.0f
	pfSegSet segset;
	float head[3];
	pfHit **hits[32];
	int isect = NULL;
	static float fall = WALL_TEST_LENGTH;
	static float vector[3];
	pfVec3 pnt, xpnt;
	pfMatrix xmat;

	if (1)	{
		segset.mode		= PFTRAV_IS_PRIM | PFTRAV_IS_CULL_BACK;
		segset.userData		= NULL;
		segset.activeMask	= 1;
		segset.isectMask	= 0x8;
		segset.bound		= NULL;
		segset.discFunc		= NULL;
         
		pfCAVEGetPosition(CAVE_HEAD,head);
	
		segset.segs[0].pos.set(head[0],head[1],head[2]);
		segset.segs[0].dir.set(xvec, yvec, zvec);
		segset.segs[0].length = WALL_TEST_LENGTH;

		isect = g_limScene->isect(&segset, hits);
    
		if (isect){
			(*hits[0])->query(PFQHIT_POINT, pnt.vec);
			(*hits[0])->query(PFQHIT_XFORM, (float*)xmat.mat);
	    
			xpnt.xformPt(pnt, xmat);
    
			vector[0] = xpnt[PF_X] - head[0];
			vector[1] = xpnt[PF_Y] - head[1];
			vector[2] = xpnt[PF_Z] - head[2];
    
			fall = sqrt(vector[0]*vector[0] + vector[1]*vector[1] + vector[2]*vector[2]);
		}
		else {
			fall = 1000;
		}

		if (fall != 1000){
			segset.segs[0].dir.set(-yvec, xvec, zvec);
			segset.segs[0].length = 0.75;

			isect = g_limScene->isect(&segset, hits);
    
			if (isect){
				fall	= 0;
				vector[0]   = -yvec / 2.0f;
				vector[2]   = xvec / 2.0f;
			}

         
			segset.segs[0].dir.set(yvec, -xvec, zvec);
			segset.segs[0].length = 0.75;

			isect = g_limScene->isect(&segset, hits);
    
			if (isect){
				fall	= 0;
				vector[0]   = yvec / 2.0f;
				vector[2]   = -xvec / 2.0f;
			}
		}
	
	}

	if (fall <= WALL_HIT_RADIUS){
		/********************************************/
		/* push the user back if they are too close */
		/********************************************/
	
		if (fall < 1){
			CAVENavTranslate(-1.5*vector[0], 0, -1.5*vector[2]);
		}
	    
		return(1);
	}
	else
		return(0);
}


// Navigate in LIMBO
void limNavigate(pfDCS *dcs)
{
	// if we are flying towards an object then we turn off
	// gravity by not trying to put our feet on the ground
	//////////////////////////////////////////////////////
	
	if (!g_batmanMode)
	    compute_nav_height();

	// Do normal CAVE navigation
	////////////////////////////
	
	float jx=CAVE_JOYSTICK_X,jy=CAVE_JOYSTICK_Y;
	if (fabs(jx) > 0.2){
		CAVENavRot(-_LIM_TURN_SPEED*jx,'z');
		g_limCurrentCaveTurn += (_LIM_TURN_SPEED * jx);
	}

	// check for people trying to walk through a wall
	checkCaveWalk();


	if (fabs(jy) > 0.2){
		float w[3];
		CAVEGetVector(CAVE_WAND_FRONT,w);


		float wdx, wdy, wdz;
    
		wdx = w[0];
		wdy = w[1];
		wdz = w[2];
	    
		wdx = (jy > 0) ? wdx : -wdx;
		wdy = (jy > 0) ? wdy : -wdy;
		wdz = (jy > 0) ? wdz : -wdz;

		int wally = 0;
		wally = checkForWall(wdx, wdy, wdz);
	    
		if (wally){
			//    fprintf(stderr, "wall\n");
		}
		else {
			//    fprintf(stderr, "clear\n");
	    
			// Lock navigation to floor
			CAVENavTranslate(w[0]*jy*_LIM_CRUISE_SPEED,
					 w[1]*jy*_LIM_CRUISE_SPEED,
					 w[2]*jy*_LIM_CRUISE_SPEED);
		}

		// Stop you from going thru the ground
		//////////////////////////////////////
	    
		float headPos[3], localHeadPos[3];
		float diff;
		CAVEGetPosition(CAVE_HEAD_NAV, headPos);
		CAVEGetPosition(CAVE_HEAD,localHeadPos);
		diff = headPos[2] - localHeadPos[2];
		if (diff < _LIM_NAV_MIN_HEIGHT)	{
			CAVENavTranslate(0,0,-diff);
		}

	}

	/* Store the CAVE navigation in the DCS */
	pfCAVEDCSNavTransform(dcs);
}

/**********************************************************************/
/* This function is supposed to deal with a user that moves themself
 * (ie by walking) through the space to try and stop that person from going
 * through a wall
 */ 
 
float g_oldCAVELoc[3] = {-1000, -1000, -1000};
float g_newCAVELoc[3];


int checkCaveWalk()
{
	float movementInCAVE[3];
	float NormmovementInCAVE[3];
	float lengthOfMovement;
    
	pfSegSet segset;
	float head[3];
	pfHit **hits[32];
	int isect = NULL;
   
	pfCAVEGetPosition(CAVE_HEAD,head);
    
	if (g_oldCAVELoc[0] == -1000){
		g_oldCAVELoc[0] = head[0];
		g_oldCAVELoc[1] = head[1];
		g_oldCAVELoc[2] = head[2];
	}

	g_newCAVELoc[0] = head[0];
	g_newCAVELoc[1] = head[1];
	g_newCAVELoc[2] = head[2];
    
	movementInCAVE[0] = g_newCAVELoc[0] - g_oldCAVELoc[0];
	movementInCAVE[1] = g_newCAVELoc[1] - g_oldCAVELoc[1];
	movementInCAVE[2] = g_newCAVELoc[2] - g_oldCAVELoc[2];
    
	lengthOfMovement = sqrt(movementInCAVE[0]*movementInCAVE[0] +
				movementInCAVE[1]*movementInCAVE[1]);

	/*********************************************************************/
	/* if the user has moved significantly from the last 'safe' location */
	/*********************************************************************/

    if (lengthOfMovement >= 0.3){
	    segset.mode		= PFTRAV_IS_PRIM | PFTRAV_IS_CULL_BACK;
	    segset.userData		= NULL;
	    segset.activeMask	= 1;
	    segset.isectMask	= 0x8;
	    segset.bound		= NULL;
	    segset.discFunc		= NULL;

	    NormmovementInCAVE[0] = movementInCAVE[0] / lengthOfMovement;
	    NormmovementInCAVE[1] = movementInCAVE[1] / lengthOfMovement;
	    NormmovementInCAVE[2] = 0;
	
	    segset.segs[0].pos.set(g_oldCAVELoc[0], g_oldCAVELoc[1], g_oldCAVELoc[2]);
	    segset.segs[0].dir.set(NormmovementInCAVE[0], NormmovementInCAVE[1], 0);
	    segset.segs[0].length = lengthOfMovement+1;

	    isect = g_limScene->isect(&segset, hits);

	    if (isect){
		    /******************************************************************/
		    /* the user is too close to a wall ... move entire CAVE backwards */
		    /******************************************************************/
	    
		    CAVENavTranslate(-movementInCAVE[0], -movementInCAVE[1], 0);
	    } else {
		    g_oldCAVELoc[0] = g_newCAVELoc[0];
		    g_oldCAVELoc[1] = g_newCAVELoc[1];
		    g_oldCAVELoc[2] = g_newCAVELoc[2];
	    }
    }
    
    return(0);
}

