/*
 * LIMBO
 * 4/3/98
 * (C) 1998 by Jason Leigh
 * Univ Illinois at Chicago
 *
 */

#include "lim_global.h++"
#include "lim_perf.h++"
#include "lim_model.h++"

pfDCS *g_limMainGroup = NULL;
pfScene *g_limScene = NULL;
pfDCS *g_limWandDCS = NULL;
pfDCS *g_limMicDCS = NULL;
pfDCS *g_sphereDCS = NULL;
pfDCS *g_arrowDCS = NULL;
pfDCS *g_selectedDCS = NULL;
int g_limObjectCount = 0;

/************************************************************************/

pfDCS * createScene(pfChannel *chan, pfScene *scene);
void loadMainSceneObjects(pfDCS *parent);
pfDCS * loadMicObject();

/************************************************************************/

// Initialize performer.
void limPerformerInit(int argc, char ** argv)
{

	// Set the CAVE's far clipping plane to very far!
	CAVEFar = 100000;

	// Initialize Performer
	pfInit();

	char *limpath = getenv("LIMBO");
	char fpath[1024];
	if (limpath)
		sprintf(fpath,"./:/usr/share/Performer/data:%s:%s/LIMBO_MODELS:%s/LIMBO_MODELS/FONTS", limpath,limpath,limpath);
	else 
		sprintf(fpath,"./:/usr/share/Performer/data:LIMBO_MODELS:LIMBO_MODELS/FONTS");
	pfFilePath(fpath);

	printf("LIMBO: Performer Search Path: %s\n", fpath);

	// Equivalent to CAVE Config. Must be called after pfInit()
	pfCAVEConfig(&argc,argv,NULL);

	// Performer config
	pfConfig();

	// Craetes all the channels for performer
	pfCAVEInitChannels();

	// Create a scene
	g_limScene = new pfScene;
	g_limMainGroup = createScene(pfCAVEMasterChan(),g_limScene);

}

// Perform one frame update
void limPerfOneCycle()
{
		// Required by performer to synch things to a certain frame rate.
      		pfSync();

		// Updates tracker data and sets up new projects for each channel.
		pfCAVEPreFrame();

		// Required by performer to trigger cull and draw
		pfFrame();

		// Updates non latency critical data.
		pfCAVEPostFrame();

}

// Create the LIMBO scene with a couple of lights and load in all the basic models.
pfDCS * createScene(pfChannel *chan, pfScene *scene)
{
	/* Create a pfScene */
	pfGeoState *gstate = new pfGeoState;
	pfDCS *group = new pfDCS;

	/* Set up the default GeoState to specify the how the geometry is to
	   be displayed.
	   */
	gstate->setMode(PFSTATE_ENLIGHTING, PF_ON);
	gstate->setMode(PFSTATE_CULLFACE, PFCF_OFF);
	scene->setGState(gstate);

	/* Add a light source to the scene as well as the dcs */
	pfLightSource *myLightSource = new pfLightSource;
	myLightSource->setPos(10,-10,10,1);

	pfLightSource *myLightSource2 = new pfLightSource;
	myLightSource2->setPos(-10,-10,10,1);


	scene->addChild(myLightSource);
	scene->addChild(myLightSource2);


	/* Add the group, and load the objects under it */
	scene->addChild(group);
	scene->addChild(loadMicObject());

	loadMainSceneObjects(group);

	void loadManipulableDummyModels(pfDCS *parent);
	loadManipulableDummyModels(group);

	/* Assign the scene to the display channel */
	chan->setScene(scene);

	return group;
}

// Create the DCS' on which the downloaded models will be attached.
// These DCS' will be labelled so that Performer can find them
// when doing a ray intersection during picking. See lim_pick.c++
void loadManipulableDummyModels(pfDCS *parent)
{
	for (int i = 0; i < _LIM_MAX_MODELS; i++) {
		pfDCS *newDCS = new pfDCS;
		char objName[256];
		sprintf(objName,"Jason %d", i);
		newDCS->setName(objName);

		newDCS->setTravMask(PFTRAV_ISECT,
				  // Intersection mask
				  0x8,
				  PFTRAV_SELF | PFTRAV_DESCEND | PFTRAV_IS_CACHE,
				  PF_SET);
		g_listOfModelDCS[i] = newDCS;
		parent->addChild(newDCS);
	}		
}

// Remove the object under the current parent.
void limPerfRemoveObject(pfDCS *parent)
{
	if (parent) {
		parent->setTrans(0,0,0);
		if (parent->getNumChildren()) {
			pfNode *aChild = parent->getChild(0);
			if (aChild)
				parent->removeChild(aChild);
		}


	}
}

// Replace the object under the current parent.
void limPerfReplaceObject(char *filename,pfDCS *parent)
{
	pfNode *obj;

	if (obj = pfdLoadFile(filename)) {
		if (parent) {
			int num = parent->getNumChildren();
			fflush(stdout);
			if (num) {
				pfNode *aChild = parent->getChild(0);
				if (aChild) {
					parent->removeChild(aChild);
					parent->addChild(obj);
				}
			} else {
				parent->addChild(obj);
			}
		}
	}
}

// Load the microphone intercom object.
pfDCS * loadMicObject()
{

	pfNode *obj;

	pfdInitConverter("LIMBO_MODELS/mic.iv");
	if (obj = pfdLoadFile("LIMBO_MODELS/mic.iv")) {

		g_limMicDCS = new pfDCS;
		g_limMicDCS->setTrans(0,-10,50000);
		g_limMicDCS->addChild(obj);
		return g_limMicDCS;
	}
	return NULL;
}

// Load the main LIMBO scene objects.
void loadMainSceneObjects(pfDCS *parent)
{
	pfNode *obj;


	// This is the floor.
	if (obj = pfdLoadFile("LIMBO_MODELS/twirl.iv")) {
		parent->addChild(obj);
	}

	// This is the sphere that appears as part of navigation.
	if (obj = pfdLoadFile("LIMBO_MODELS/sphere.iv")) {
              	g_sphereDCS = new pfDCS;
		g_sphereDCS->setTrans(0,-10,-2);
		g_sphereDCS->addChild(obj);
		parent->addChild(g_sphereDCS);
	}

	// This is a set of arrows that appears when objects are picked and moved.
	if (obj = pfdLoadFile("LIMBO_MODELS/arrow.iv")) {
              	g_arrowDCS = new pfDCS;
		g_arrowDCS->setTrans(0,-10,-2);
		g_arrowDCS->addChild(obj);
		parent->addChild(g_arrowDCS);
	}

	// This is a blinking cursor.
	if (obj = pfdLoadFile("LIMBO_MODELS/box.iv")) {
              	g_selectedDCS = new pfDCS;
		g_selectedDCS->setTrans(0,-10,-2);
		g_selectedDCS->addChild(obj);
		parent->addChild(g_selectedDCS);
	}

	// This is the LIMBO logo.
	if (obj = pfdLoadFile("LIMBO_MODELS/limbo.iv")) {
              	pfDCS *limboDCS = new pfDCS;
		limboDCS->addChild(obj);
		parent->addChild(limboDCS);
	}

	// This is your wand pointer.
	if (obj = pfdLoadFile("LIMBO_MODELS/wand.iv")) {
		g_limWandDCS = new pfDCS;
		g_limWandDCS->setTravMask(PFTRAV_ISECT,
				  // Intersection mask
				  0x0,
				  PFTRAV_SELF | PFTRAV_DESCEND | PFTRAV_IS_CACHE,
				  PF_OR);

		g_limWandDCS->addChild(obj);

		parent->addChild(g_limWandDCS);
	}
}


// Update the hand DCS with the CAVE's current wand position and orientation.
void limUpdateWand()
{
	float wandP[3], wandO[3];

	pfCAVEGetPosition(CAVE_WAND_NAV, wandP);
	pfCAVEGetOrientation(CAVE_WAND_NAV,wandO);
	g_limWandDCS->setTrans(wandP[0], wandP[1], wandP[2]);
	g_limWandDCS->setRot(wandO[2],wandO[0],wandO[1]);

}

// Filename is usually suffixed with .pfb
void limExport(pfNode *anode, char *filename)
{
	pfdStoreFile(anode, filename);
}
