/*
 * LIMBO
 * 4/3/98
 * (C) 1998 by Jason Leigh
 * Univ Illinois at Chicago
 *
 * Pick, move and rotate an object.
 *
 */

#include "lim_global.h++"
#include "lim_perf.h++"
#include "lim_model.h++"

// The picking ray is about 10000 feet.
#define _LIM_PICKING_RAY_LENGTH 10000

/************************************************************************/

// Calls to show and hide objects that  indicate the state of the
// picking operation.
void hideSelectedDCS()
{
	g_selectedDCS->setTrans(0,-10,-2);
}

void hideSphereDCS()
{
	g_sphereDCS->setTrans(0,-10,-2);
}

void hideArrowDCS()
{
	g_arrowDCS->setTrans(0,-10,-2);
}

void showSelectedDCS(float x, float y, float z)
{
	g_selectedDCS->setTrans(x,y,z);
}


void showSphereDCS(float x, float y, float z)
{
	g_sphereDCS->setTrans(x,y,z);
}

void showArrowDCS(float x, float y, float z)
{
	g_arrowDCS->setTrans(x,y,z);
}


// Andy
// from Dave's grabber class
// danger, do not stare directly into the mathematics

void GetWandMatrix(pfMatrix& m)
{
 float or[3],pos[3];
 CAVEGetPosition(CAVE_WAND_NAV,pos); 
 CAVEGetOrientation(CAVE_WAND_NAV,or);
 m.makeTrans(pos[0],pos[1],pos[2]);
 m.preRot(or[2],0.0,0.0,1.0,m);
 m.preRot(or[0],1.0,0.0,0.0,m);
 m.preRot(or[1],0.0,1.0,0.0,m);
}

// Andy
// from Dave's grabber class
// danger, do not stare directly into the mathematics

void GetInverseWandMatrix(pfMatrix& m)
{
 float or[3],pos[3];
 CAVEGetPosition(CAVE_WAND_NAV,pos); 
 CAVEGetOrientation(CAVE_WAND_NAV,or);
 m.makeRot(-or[1],0.0,1.0,0.0);
 m.preRot(-or[0],1.0,0.0,0.0,m);
 m.preRot(-or[2],0.0,0.0,1.0,m);
 m.preTrans(-pos[0],-pos[1],-pos[2],m);
}





// Handle the selection and movement of objects.
void limHandleSelectAndMove(pfScene *scene)
{
	static pfDCS *item = NULL;
	static pfDCS *navItem = NULL;
	pfDCS *pickedItem;
	static int itemNum = 0;
	static int navItemNum = 0;
	int selectedItemNum = 0;
	float wandPos[3], wandVec[3];
	static float cenX, cenY, cenZ;

	// Andy - more stuff from Dave's grabber class
	static pfMatrix grabMat, arMat;


	static float mag;

	pfSphere bsp;
	float newDat[3];
	float hitLocation[3], worldHitLocation[3];
	static float lockedInHitLocation[3];

	pfDCS * doSelect(pfScene *, int*, float*);

	// Andy
	extern int g_batmanMode;


	// If left & middle button pressed then hide
	// all cursors so that we can cleanly figure out
	// what in the scene can be picked.
	if (CAVEBUTTON1 && CAVEBUTTON2) {
		hideSphereDCS();
		hideArrowDCS();
		hideSelectedDCS();
	}

	// Figure out what object has been selected...
	pickedItem = doSelect(scene, &selectedItemNum, hitLocation);

	// Draw a cursor near the object that has been selected.
	if (pickedItem) {
		CAVEGetPosition(CAVE_WAND_NAV, wandPos);
		CAVENavConvertCAVEToWorld(hitLocation,worldHitLocation);
		showSelectedDCS(worldHitLocation[0],
				worldHitLocation[1],
				worldHitLocation[2]);
	} else hideSelectedDCS();


	// Once the object is found, if left and middle button is 
	// still selected then remove the object from the scene.
	if (CAVEBUTTON1 && CAVEBUTTON2) {
		hideSphereDCS();
		hideArrowDCS();
		hideSelectedDCS();

		// Tell performer to do an intersection of a ray with the scene.
		if (pickedItem) {
			limMarkModelForRemoval(itemNum);
		}
		return;
	}


	// If left button pressed:
	if (CAVEBUTTON1) {

		hideSelectedDCS();


		// If there is no item already in your hand then try and find one.
		if (item == NULL) {

			// Tell performer to do an intersection of a ray with the scene.
			item = doSelect(scene, &itemNum, hitLocation);

			// When an object has been selected figure out
			// what the current starting wand position is and
			// figure out where the exact point of intersection of the
			// wand ray to the selected object is.
			if (item) {
				CAVEGetPosition(CAVE_WAND_NAV, wandPos);

				// Grab the transform matrix of the selected object.
				pfMatrix mat;
				item->getMat(mat); float w;

				// Extract translation component of the performer object.
				mat.getRow(3, &cenX, &cenY, &cenZ, &w);

				// Exact intersect of wand vector to object in world coord.
				CAVENavConvertCAVEToWorld(hitLocation,lockedInHitLocation);

				// Draw the picking arrow.
				showArrowDCS(lockedInHitLocation[0],
						      lockedInHitLocation[1],
						      lockedInHitLocation[2]);

				// Distance of intersect to wand.
				mag= fsqrt(((lockedInHitLocation[0] - wandPos[0]) *
				      (lockedInHitLocation[0] - wandPos[0])) +
					((lockedInHitLocation[1] - wandPos[1]) *
				      (lockedInHitLocation[1] - wandPos[1])) +
					((lockedInHitLocation[2] - wandPos[2]) *
				      (lockedInHitLocation[2] - wandPos[2])));

				// more stuff from dave's grabber DCS class
				grabMat.makeIdent();
				item->getMat(grabMat);
				pfMatrix invmat;
				GetInverseWandMatrix(invmat);
				grabMat *= invmat;

				// need to move the arrows icon as well
				arMat.makeIdent();
				g_arrowDCS->getMat(arMat);
				arMat *= invmat;
			}

		} else  {
		
			// Andy
			// here I'm going to take a bunch of code from Dave's grabber DCS
			// class that does all kinds of matrix stuff that should work here :)
			//
			// this combines the moving and rotation under a single button 

			pfMatrix sensMat, newMat, newMat2, arrowMat;
			GetWandMatrix(sensMat);
		    
			newMat = grabMat * sensMat;
			item->setMat(newMat);

		    // now set the location of the icon
			newMat2 = arMat * sensMat;
			g_arrowDCS->setMat(newMat2);


			float thaMatrix[16];
			pfMatrix objMat;
			item->getMat(objMat);
			objMat.getRow(0, &thaMatrix[0],  &thaMatrix[1],  &thaMatrix[2],  &thaMatrix[3]);
			objMat.getRow(1, &thaMatrix[4],  &thaMatrix[5],  &thaMatrix[6],  &thaMatrix[7]);
			objMat.getRow(2, &thaMatrix[8],  &thaMatrix[9],  &thaMatrix[10], &thaMatrix[11]);
			objMat.getRow(3, &thaMatrix[12], &thaMatrix[13], &thaMatrix[14], &thaMatrix[15]);

			// Place wand data into Me key for the object to distribute this
			// value to everyone.
			CAVERN_irbKey_c *aKey = g_limListOfModelMetaDataKeys[itemNum];
			CAVERN_irbKey_c::status_t keyStatus;
			if (aKey) {
				int size = sizeof(float) * 16; //3
				aKey->put((char*) &objMat, &size, &keyStatus);
			}

		}
	}

	// If button 1 released release the object being moved.
	if (!CAVEBUTTON1) {
		item = NULL;
		itemNum = 0;
		hideArrowDCS();
	}


	// If button 2 hit then lock down the hit position so that we can 
	// subsequently interpolate towards the object.
	if (CAVEBUTTON2) {

		hideSelectedDCS();

		if (navItem == NULL) {
			navItem = doSelect(scene, &navItemNum, hitLocation);
			if (navItem)
			    {
			    lockedInHitLocation[0] = hitLocation[0];
			    lockedInHitLocation[1] = hitLocation[1];
			    lockedInHitLocation[2] = hitLocation[2];

			    float realHitLocation[3];

			    // Exact intersect of wand vector to object in world coord.
			    CAVENavConvertCAVEToWorld(hitLocation,realHitLocation);

			    showSphereDCS(realHitLocation[0],
						  realHitLocation[1],
						  realHitLocation[2]);
			}
		}
	}

	// If button 2 released then release the item we are interpolating towards.
	if (!CAVEBUTTON2){
		navItem = NULL;
		hideSphereDCS();
	}

	// We have an object we are interpolating towards.
	if (navItem) {
		float bodyPos[3];
		float deltaX, deltaY, deltaZ;
		CAVEGetPosition(CAVE_HEAD,bodyPos);
		deltaX = lockedInHitLocation[0] - bodyPos[0];
		deltaY = lockedInHitLocation[1] - bodyPos[1];
		deltaZ = lockedInHitLocation[2] - bodyPos[2];
		CAVENavTranslate(deltaX/20.0, deltaY/20.0, deltaZ/20.0);

		// enable batman mode ... turn off gravity while moving towards an object
		// Andy
		g_batmanMode = 1;

		{
			// Make sure you can't navigate below the
			// ground.
			float headPos[3], localHeadPos[3];
			float diff;
			CAVEGetPosition(CAVE_HEAD_NAV, headPos);
			CAVEGetPosition(CAVE_HEAD,localHeadPos);
			diff = headPos[2] - localHeadPos[2];
			if (diff < 0) {
				CAVENavTranslate(0,0,-diff);
			}
		}
		
		lockedInHitLocation[0] -= (deltaX/20.0);
		lockedInHitLocation[1] -= (deltaY/20.0);
		lockedInHitLocation[2] -= (deltaZ/20.0);
	}
	else {
		// Andy
		g_batmanMode = 0;   
	}
	
}

/* Go thru the array of intersections and take a look at each intersection */
pfNode * dealWithPicking(pfHit **hits[32], float hitLocation[3])
{
    int numObjects;
    int i;
    pfMatrix transformMatrix;
    pfVec3 transformedIsectPoint, isectPoint;

    // Get the path of only the first intersection.
    pfPath * path = NULL;
    (*hits[0])->query(PFQHIT_PATH, &path);


    // Figure out the point of intersection.
    (*hits[0])->query(PFQHIT_POINT, isectPoint.vec);

    (*hits[0])->query(PFQHIT_XFORM, (float*)transformMatrix.mat);

    transformedIsectPoint.xformPt(isectPoint, transformMatrix);
    hitLocation[0] = transformedIsectPoint[PF_X];
    hitLocation[1] = transformedIsectPoint[PF_Y];
    hitLocation[2] = transformedIsectPoint[PF_Z];

    /*
    hitLocation[0] = isectPoint[PF_X];
    hitLocation[1] = isectPoint[PF_Y];
    hitLocation[2] = isectPoint[PF_Z];
    */
    pfNode * testIt;

    // Determine how many nodes are in the path
    numObjects = path->getNum();

    // Go thru each node in the path and see if it's name is "Jason".
    for(i=0; i<numObjects; i++) {
	    testIt = (pfNode *) path->get(i);
	
	    if (testIt != NULL) {
		    if (testIt->getName() != NULL) {
			    if (strncmp(testIt->getName(), "Jason", 5) == 0) {
				    return testIt;
			    }
		    }
	    }
    }

    return NULL;

}

/* Given a scene, do the selection by creating a performer ray with which
 to intersect the scene.
 */
pfDCS * doSelect(pfScene *hierarchy, int *itemNum, float hitLocation[3])
{
    pfSegSet segset;
    float wandVec[3];
    pfHit **hits[32];
    int isect = NULL;
    float rayStart[3];
 
    // set up for intersection testing


    segset.mode		= PFTRAV_IS_PATH | PFTRAV_IS_GSET | PFTRAV_IS_CULL_BACK;
    segset.userData	= NULL;
    segset.activeMask	= 1;
    segset.isectMask	= 0x8;  // Intersection mask
    segset.bound	= NULL;
    segset.discFunc	= NULL;
     
    // need vector to start after the object currently in hand

    CAVEGetPosition(CAVE_WAND,rayStart);
    CAVEGetVector(CAVE_WAND_FRONT,wandVec);


    // Set the location of the picking ray and the ray vector and its length
    segset.segs[0].pos.set(rayStart[0], rayStart[1], rayStart[2]);
    segset.segs[0].dir.set(wandVec[0], wandVec[1], wandVec[2]);
    segset.segs[0].length = _LIM_PICKING_RAY_LENGTH; // Length of the picking ray

    // see what the beam from the wand intersects

    // Hits contains an array of intersections.
    isect = hierarchy->isect(&segset, hits);

    pfDCS *selectedDCS;

    // If it has intersected something see if it is
    // something that has a mask of 0x8- ie our loaded objects.
    if (isect) {
	    selectedDCS = (pfDCS *) dealWithPicking(hits, hitLocation);

	    if (selectedDCS) {
		    // Extract the name of the DCS (1st string is "Jason")
		    // next value is the object number.
		    char dummy[12];
		    sscanf(selectedDCS->getName(),"%s %d",dummy, itemNum);
	    }
	    return selectedDCS;
    }
    return NULL;
		
}
