/*
 * LIMBO
 * (C) 1998 by Jason Leigh
 * Univ Illinois at Chicago

 Use this to distribute your own custom data.
 2 channels are created with the call: limInitUserData().- called in lim_init.c++
 One channel is reliable, the other unreliable.

 Use limUserReliableSend() to send data over the reliable channel.
 Use limUserUnReliableSend() to send data over the unreliable channel.
 Modify userReliableCB() and userUnreliableCB() below to handle the situation
 when incoming data arrives.
 */

#include "lim_global.h++"
/// CONSOLE
/// #include "lim_init.h++"
#include "console_global.h++"
#include "console_MsgMgr.h++"
//#include "lim_console.h++"
/// CONSOLE

CAVERN_irbChannel_c *g_limUserReliableChannel = NULL;
CAVERN_irbKey_c *g_limUserReliableKey = NULL;
CAVERN_irbLink_c *g_limUserReliableLink = NULL;

CAVERN_irbChannel_c *g_limUserUnReliableChannel = NULL;
CAVERN_irbKey_c *g_limUserUnReliableKey = NULL;
CAVERN_irbLink_c *g_limUserUnReliableLink = NULL;


// Use this call to send your own data over a reliable channel.
void limUserReliableSend(char *data, int size)
{
	CAVERN_irbKey_c::status_t status;
        cvrnPrintf("Sending reliable user data\n");
	g_limUserReliableKey->put(data, &size, &status);
}

// Use this call to send your own data over an unreliable channel.
void limUserUnReliableSend(char *data, int size)
{
	CAVERN_irbKey_c::status_t status;
	g_limUserUnReliableKey->put(data, &size, &status);
}

// Edit this to intercept any incoming data over the reliable channel.
void userReliableCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData)
{
	int size;
	int maxsize = 1024;
	char data[1024];

	CAVERN_irbKey_c::status_t status;
	thisKey->getSize(&size, &status);
	CAVERNplus_datapack_c datapack;


	if (size) {
		thisKey->get(data, &size, &status);
	        datapack.setBuffer(data, size);
	        int command = datapack.unpackInt();
		cvrnPrintf("Incoming reliable user data\n");
		cvrnPrintf("%d\n", command);
                g_ConsoleMessageMgr->clientReceive(command, &datapack); 
	} else {
		cvrnPrintf("Incoming reliable data size 0\n");
	}
}

// Edit this to intercept any incoming data over the unreliable channel.
void userUnReliableCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData)
{
	int size;
	int maxsize = 1024;
	char data[1024];

	CAVERN_irbKey_c::status_t status;
	thisKey->getSize(&size, &status);
	CAVERNplus_datapack_c datapack;

	if (size) {
		thisKey->get(data, &maxsize, &status);
	        datapack.setBuffer(data, size);
	        int command = datapack.unpackInt();
		cvrnPrintf("Incoming Unreliable user data\n");
		cvrnPrintf("%d\n", command);
                g_ConsoleMessageMgr->clientReceive(command, &datapack); 
	} else {
		cvrnPrintf("Incoming Unreliable data size 0\n");
	}
}

// -------------------------------------------------------------------

void createUserChannels(char *remoteHost)
{
	g_limUserReliableChannel = g_limPIRB->createChannel();
	g_limUserUnReliableChannel = g_limPIRB->createChannel();

	CAVERN_irbChannel_c::status_t channelRetStatus;

	CAVERN_irbId_c remoteIRBId;
	remoteIRBId.setAddress(remoteHost);
	remoteIRBId.setPort(g_limRemotePort);

	do {
		cvrnPrintf("Reliable User Channel: Attempting to Connect...\n");
		g_limUserReliableChannel->open(&remoteIRBId,
                                               NULL,
                                               CAVERN_irbChannel_c::RELIABLE,
		                               &channelRetStatus);
		sleep(1);
	} while(channelRetStatus != CAVERN_irbChannel_c::OK);

	cvrnPrintf("Reliable User Channel: Connected\n");

	do {
		cvrnPrintf("UnReliable User Channel: Attempting to Connect...\n");
		g_limUserUnReliableChannel->open(&remoteIRBId,
                                                 NULL,
                                                CAVERN_irbChannel_c::UNRELIABLE,
		                                 &channelRetStatus);
		sleep(1);
	} while(channelRetStatus != CAVERN_irbChannel_c::OK);

	cvrnPrintf("UnReliable User Channel: Connected\n");


}


// Make a link to link the USER_RELIABLE_DATA_KEY and USER_UNRELIABLE_DATA_KEY
void makeUserLinks()
{
	CAVERN_irbKeyId_c userReliableKeyId, userUnReliableKeyId;

	userReliableKeyId.setName("USER_RELIABLE_DATA_KEY");
	userReliableKeyId.setPath(g_worldName);

	if (g_limUserReliableChannel) {

		if (g_limUserReliableLink) delete g_limUserReliableLink;

		CAVERN_linkAttrib_c linkAttribute;

		linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::NONE);
		g_limUserReliableLink = 
                    g_limUserReliableChannel->link(g_limUserReliableKey, 
                                                   &userReliableKeyId,
                                                   &linkAttribute);
		if (g_limUserReliableLink == NULL) {
			cvrnPrintf("Reliable User Channel: The link didn't work!\n");
		}

	}


	userUnReliableKeyId.setName("USER_UNRELIABLE_DATA_KEY");
	userUnReliableKeyId.setPath(g_worldName);

	if (g_limUserUnReliableChannel) {

		if (g_limUserUnReliableLink) delete g_limUserUnReliableLink;

		CAVERN_linkAttrib_c linkAttribute;

		linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::NONE);
		g_limUserUnReliableLink = 
                    g_limUserUnReliableChannel->link(g_limUserUnReliableKey, 
                                                     &userUnReliableKeyId,
                                                     &linkAttribute);
		if (g_limUserUnReliableLink == NULL) {
			cvrnPrintf("UnReliable User Channel: The link didn't work!\n");
		}

	}
}

/// CONSOLE
CAVERN_irbChannel_c *g_limServerReliableChannel = NULL;
CAVERN_irbKey_c *g_limServerReliableKey = NULL;
CAVERN_irbLink_c *g_limServerReliableLink = NULL;

// Use this call to send server data over a reliable channel.
void limServerReliableSend(char *data, int size)
{
	CAVERN_irbKey_c::status_t status;
	g_limServerReliableKey->put(data, &size, &status);
}

// Edit this to intercept any incoming data over the server channel.
void userServerCB(CAVERN_irbKey_c::CAVERN_irbKeyEvent_t event,
	     CAVERN_irbKey_c *thisKey,
	     void* userData)
{
	int size;
	int maxsize = 1024;
	char data[1024];

	CAVERN_irbKey_c::status_t status;
	thisKey->getSize(&size, &status);
	CAVERNplus_datapack_c datapack;

	if (size) {
		thisKey->get(data, &maxsize, &status);
	        datapack.setBuffer(data, size);
	        char command = datapack.unpackChar();
		cvrnPrintf("Incoming server data\n");
		cvrnPrintf("%c\n", command);
                g_ConsoleMessageMgr->serverReceive(command, &datapack); 
	} else {
		cvrnPrintf("Incoming Server data size 0\n");
	}
}

void createServerChannels(char *remoteHost)
{
	g_limServerReliableChannel = g_limPIRB->createChannel();
	CAVERN_irbChannel_c::status_t channelRetStatus;

	CAVERN_irbId_c remoteIRBId;
	remoteIRBId.setAddress(remoteHost);
	remoteIRBId.setPort(g_limRemotePort);

	do {
		cvrnPrintf("Server Channel: Attempting to Connect...\n");
		g_limServerReliableChannel->open(&remoteIRBId,
                                            NULL,
                                            CAVERN_irbChannel_c::RELIABLE,
		                            &channelRetStatus);
		sleep(1);
	} while(channelRetStatus != CAVERN_irbChannel_c::OK);

	cvrnPrintf("Server Channel: Connected\n");
}

void makeServerLinks()
{
	CAVERN_irbKeyId_c serverReliableKeyId;

	serverReliableKeyId.setName("SERVER_RELIABLE_DATA_KEY");
	serverReliableKeyId.setPath(g_worldName);

	if (g_limServerReliableChannel) {

		if (g_limServerReliableLink) delete g_limServerReliableLink;

		CAVERN_linkAttrib_c linkAttribute;

		linkAttribute.setInitialSynchType(CAVERN_linkAttrib_c::NONE);
		g_limServerReliableLink = 
                      g_limServerReliableChannel->link(g_limServerReliableKey, 
                                                       &serverReliableKeyId,
                                                       &linkAttribute);
		if (g_limServerReliableLink == NULL) {
			cvrnPrintf("ServerReliable User Channel: The link didn't work!\n");
		}

	}


}
/// CONSOLE

// Initialize channels and links to allow you to send your own data.
// This is called in limInit() in lim_init.c++
void limUserDataInit()
{
	CAVERN_irb_c::status_t irbStatus;
	CAVERN_irbKeyId_c userReliableKeyId, userUnReliableKeyId;


        /// CONSOLE
	CAVERN_irbKeyId_c serverReliableKeyId;

	serverReliableKeyId.setName("SERVER_RELIABLE_DATA_KEY");
	serverReliableKeyId.setPath(g_worldName);
	g_limServerReliableKey = g_limPIRB->define(&serverReliableKeyId,
                                                     NULL, &irbStatus);
	g_limServerReliableKey->trigger(userServerCB, (void *) NULL);
        /// CONSOLE


	userReliableKeyId.setName("USER_RELIABLE_DATA_KEY");
	userReliableKeyId.setPath(g_worldName);

	g_limUserReliableKey = g_limPIRB->define(&userReliableKeyId, 
                                                 NULL, &irbStatus);
	g_limUserReliableKey->trigger(userReliableCB, (void *) NULL);

	userUnReliableKeyId.setName("USER_UNRELIABLE_DATA_KEY");
	userUnReliableKeyId.setPath(g_worldName);

	g_limUserUnReliableKey = g_limPIRB->define(&userUnReliableKeyId, 
                                                   NULL, &irbStatus);
	g_limUserUnReliableKey->trigger(userUnReliableCB, (void *) NULL);

	if (g_limRemoteConnect) {
		// Open the channel to a remote IRB.
		createUserChannels(g_worldServer);
		// Create the links between the keys.
		makeUserLinks();
                /// CONSOLE
                createServerChannels(g_worldServer);
                makeServerLinks();
                /// CONSOLE
	}
        

}





